/* Scrutari - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.request.ParameterErrorMessageKeys;
import fr.exemole.sctserver.servlets.admin.contentstreams.LexiesOdsStream;
import fr.exemole.sctserver.servlets.admin.contentstreams.MotsclesOdsStream;
import fr.exemole.sctserver.servlets.admin.contentstreams.URITxtStream;
import fr.exemole.sctserver.servlets.admin.htmlpages.AdminHtmlPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.ErrorPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.FichesPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.IndexPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.LexiesPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.MotsclesPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.ScrutariJsPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.SearchLogPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.SearchLogsPage;
import fr.exemole.sctserver.servlets.admin.htmlpages.StatsPage;
import fr.exemole.sctserver.tools.EngineUtils;
import java.text.ParseException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.request.RequestConstants;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public final class AdminResponseHandlerFactory {

    private final ScrutariSession scrutariSession;
    private final SctEngine engine;
    private boolean notFound = false;
    private short responseHandlerType;
    private Parameters parameters;
    private final Lang workingLang;
    private CommandMessage commandMessage;
    private FuzzyDate journalDate;

    private AdminResponseHandlerFactory(SctEngine engine, String relativePath, RequestMap requestMap) {
        this.engine = engine;
        this.workingLang = EngineUtils.getDefaultLang(engine);
        this.scrutariSession = engine.getScrutariSession();
        if (relativePath.equals("")) {
            initAndDoCommand(requestMap);
        } else if (relativePath.equals("index.html")) {
            responseHandlerType = AdminConstants.INDEX_PAGE;
        } else if (relativePath.equals("stats.html")) {
            responseHandlerType = AdminConstants.STATS_PAGE;
        } else if (relativePath.equals("fiches.html")) {
            responseHandlerType = AdminConstants.FICHES_PAGE;
        } else if (relativePath.equals("lexies.html")) {
            responseHandlerType = AdminConstants.LEXIES_PAGE;
        } else if (relativePath.equals("uri.txt")) {
            responseHandlerType = AdminConstants.URI_STREAM;
        } else if (relativePath.equals("searchlogs.html")) {
            responseHandlerType = AdminConstants.SEARCHLOGS_PAGE;
        } else if (relativePath.equals("scrutarijs.html")) {
            responseHandlerType = AdminConstants.SCRUTARIJS_PAGE;
        } else if (relativePath.startsWith("motscles")) {
            parameters = new Parameters();
            if (!parameters.init(relativePath.substring("motscles".length()), false)) {
                setNotFound();
                return;
            }
            String extension = parameters.getExtension();
            if (extension.equals("html")) {
                responseHandlerType = AdminConstants.MOTSCLES_PAGE;
            } else if (extension.equals("ods")) {
                responseHandlerType = AdminConstants.MOTSCLES_STREAM;
            } else {
                setNotFound();
            }
        } else if (relativePath.startsWith("lexies_")) {
            parameters = new Parameters();
            if (!parameters.init(relativePath.substring("lexies_".length()), true)) {
                setNotFound();
                return;
            }
            String extension = parameters.getExtension();
            if (extension.equals("html")) {
                responseHandlerType = AdminConstants.LEXIES_PAGE;
            } else if (extension.equals("ods")) {
                responseHandlerType = AdminConstants.LEXIES_STREAM;
            } else {
                setNotFound();
            }
        } else if (relativePath.startsWith("searchlog_")) {
            JournalParameters journalParameters = new JournalParameters();
            if (!journalParameters.init(relativePath.substring("searchlog_".length()))) {
                setNotFound();
                return;
            }
            String extension = journalParameters.getExtension();
            if (extension.equals("html")) {
                responseHandlerType = AdminConstants.SEARCHLOG_PAGE;
                journalDate = journalParameters.getDate();
            } else {
                setNotFound();
            }
        } else {
            setNotFound();
        }
    }

    public static ResponseHandler getHandler(SctEngine engine, String relativePath, RequestMap requestMap) {
        AdminResponseHandlerFactory factory = new AdminResponseHandlerFactory(engine, relativePath, requestMap);
        if (factory.notFound) {
            return null;
        } else {
            return factory.getResponseHandler();
        }
    }

    private ResponseHandler getResponseHandler() {
        switch (responseHandlerType) {
            case AdminConstants.LEXIES_STREAM:
                return new LexiesOdsStream(scrutariSession, workingLang, parameters.getPageLang());
            case AdminConstants.URI_STREAM:
                return new URITxtStream(scrutariSession);
            case AdminConstants.MOTSCLES_STREAM:
                return new MotsclesOdsStream(scrutariSession, workingLang);
            default:
                AdminHtmlPage page = getAdminHtmlPage();
                page.setCurrentPage(responseHandlerType);
                page.setCommandMessage(commandMessage);
                return page;

        }
    }

    private AdminHtmlPage getAdminHtmlPage() {
        switch (responseHandlerType) {
            case AdminConstants.INDEX_PAGE:
                return new IndexPage(engine, workingLang);
            case AdminConstants.ERROR_PAGE:
                return new ErrorPage(engine, workingLang);
            case AdminConstants.STATS_PAGE:
                return new StatsPage(engine, workingLang);
            case AdminConstants.FICHES_PAGE:
                return new FichesPage(engine, workingLang);
            case AdminConstants.MOTSCLES_PAGE:
                return new MotsclesPage(engine, workingLang);
            case AdminConstants.LEXIES_PAGE:
                return new LexiesPage(engine, workingLang, (parameters != null) ? parameters.getPageLang() : null);
            case AdminConstants.SEARCHLOGS_PAGE:
                return new SearchLogsPage(engine, workingLang);
            case AdminConstants.SEARCHLOG_PAGE:
                return new SearchLogPage(engine, workingLang, journalDate);
            case AdminConstants.SCRUTARIJS_PAGE:
                return new ScrutariJsPage(engine, workingLang);
            default:
                throw new SwitchException("responseHandlerType=" + responseHandlerType);
        }
    }

    private void initAndDoCommand(RequestMap requestMap) {
        String cmd = requestMap.getParameter(RequestConstants.COMMAND_PARAMETER);
        if (cmd == null) {
            responseHandlerType = AdminConstants.INDEX_PAGE;
        } else {
            Command command = Command.getCommandInstance(cmd, engine);
            if (command == null) {
                responseHandlerType = AdminConstants.ERROR_PAGE;
                commandMessage = CommandMessage.error(ParameterErrorMessageKeys.UNKNOWN_PARAMETER_VALUE, RequestConstants.COMMAND_PARAMETER, cmd);
            } else {
                responseHandlerType = AdminConstants.INDEX_PAGE;
                commandMessage = command.doCommand(requestMap);
            }
        }
    }

    public static String pageToString(short pageType) {
        switch (pageType) {
            case AdminConstants.INDEX_PAGE:
                return "index";
            case AdminConstants.STATS_PAGE:
                return "stats";
            case AdminConstants.FICHES_PAGE:
                return "fiches";
            case AdminConstants.MOTSCLES_PAGE:
                return "motscles";
            case AdminConstants.LEXIES_PAGE:
                return "lexies";
            case AdminConstants.SEARCHLOGS_PAGE:
                return "searchlogs";
            case AdminConstants.SCRUTARIJS_PAGE:
                return "scrutarijs";
            default:
                return null;
        }
    }

    public static String getPageMessageKey(short pageType) {
        switch (pageType) {
            case AdminConstants.INDEX_PAGE:
                return "_ link.admin.index";
            case AdminConstants.STATS_PAGE:
                return "_ link.admin.stats";
            case AdminConstants.FICHES_PAGE:
                return "_ link.admin.fiches";
            case AdminConstants.MOTSCLES_PAGE:
                return "_ link.admin.motscles";
            case AdminConstants.LEXIES_PAGE:
                return "_ link.admin.lexies";
            case AdminConstants.SEARCHLOGS_PAGE:
                return "_ link.admin.searchlogs";
            case AdminConstants.SCRUTARIJS_PAGE:
                return "_ link.admin.scrutarijs";
            default:
                return null;
        }
    }

    private void setNotFound() {
        notFound = true;
    }


    private class JournalParameters {

        private String extension;
        private FuzzyDate date;

        JournalParameters() {
        }

        boolean init(String pathInfo) {
            int idx = pathInfo.lastIndexOf(".");
            if (idx == -1) {
                return false;
            }
            extension = pathInfo.substring(idx + 1);
            pathInfo = pathInfo.substring(0, idx);
            try {
                date = FuzzyDate.parse(pathInfo);
            } catch (ParseException pe) {
                return false;
            }
            return true;
        }

        public String getExtension() {
            return extension;
        }

        public FuzzyDate getDate() {
            return date;
        }

    }


    private class Parameters {

        private String extension;
        private Lang pageLang;

        Parameters() {
        }

        boolean init(String pathInfo, boolean testLangue) {
            int idx = pathInfo.lastIndexOf(".");
            if (idx == -1) {
                return false;
            }
            extension = pathInfo.substring(idx + 1);
            pathInfo = pathInfo.substring(0, idx);
            if (testLangue) {
                try {
                    pageLang = Lang.parse(pathInfo);
                } catch (ParseException pe) {
                    return false;
                }
            }
            return true;
        }

        public String getExtension() {
            return extension;
        }

        public Lang getPageLang() {
            return pageLang;
        }

    }

}
