/* Scrutari - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.api.AttributeDefManager;
import fr.exemole.sctserver.api.ConfState;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.api.conf.ConfConstants;
import fr.exemole.sctserver.api.log.SctLogConstants;
import fr.exemole.sctserver.servlets.admin.ConfUpdateCommand;
import fr.exemole.sctserver.servlets.admin.RestartCommand;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.AttributesPrint;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.CategoriesPrint;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.FieldsPrint;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.MetadataPrint;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.SourcesPrint;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.UriCodesPrint;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.ValidationPrint;
import java.io.IOException;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.CB;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.CommandBoxParametersBuilder;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.html.HtmlConstants;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.logging.CategorizedMessageEntry;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.MessageLog;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.PertinencePonderation;


/**
 *
 * @author Vincent Calame
 */
public class IndexPage extends AdminHtmlPage {

    private final static HtmlAttributes ERROR_ATTR = HA.classes("scrutari-admin-ErrorMessage");
    private final ConfState confState;

    public IndexPage(SctEngine engine, Lang workingLang) {
        super(engine, workingLang);
        confState = engine.getConfState();
        CommandBoxParametersBuilder commandBoxParametersBuilder = new CommandBoxParametersBuilder();
        commandBoxParametersBuilder.setCommandKeyDivAttributes(HA.classes("scrutari-admin-CommandKey"));
        commandBoxParametersBuilder.setFormAttributes(HA.action(".").method(HtmlConstants.GET_METHOD));
        commandBoxParametersBuilder.setSubmitDivAttributes(HA.classes("scrutari-admin-Submit"));
        setCommandBoxParameters(commandBoxParametersBuilder.toCommandBoxParameters());
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        startBSContainer();
        printMessage();
        printUpdateProcess();
        printConfState();
        endBSContainer();
        end();
    }

    private void printUpdateProcess() {
        startBSRow();
        this
                .H2()
                .localize("_ title.sctconf.updateprocess")
                ._H2();
        startPanel();
        printLog();
        printRestartForm();
        endPanel();
        endBSRow();
    }

    private void printLog() {
        String dataLog = engine.getEngineStorage().getLogStorage().getLog(SctLogConstants.UPDATE_LOG, "data.xml");
        if (dataLog == null) {
            return;
        }
        MessageLog messageLog = LogUtils.readMessageLog(dataLog);
        List<MessageLog.LogGroup> logGroupList = messageLog.getLogGroupList();
        if (logGroupList.isEmpty()) {
            return;
        }
        this
                .P()
                .SPAN(ERROR_ATTR)
                .localize("_ info.admin.updateprocesslog")
                ._SPAN()
                ._P();
        this
                .UL();
        for (MessageLog.LogGroup logGroup : logGroupList) {
            this
                    .LI();
            this
                    .P()
                    .CODE()
                    .escape('[')
                    .escape(logGroup.getURI())
                    .escape(']')
                    ._CODE()
                    ._P();
            List<CategorizedMessageEntry> messageList = logGroup.getEntryList();
            int messageCount = messageList.size();
            int maxMessageCount = messageCount;
            boolean withMax = false;
            if (messageCount > 10) {
                maxMessageCount = 5;
                withMax = true;
            }
            this
                    .UL();
            for (int j = 0; j < maxMessageCount; j++) {
                CategorizedMessageEntry entry = messageList.get(j);
                String category = entry.getCategory();
                this
                        .LI()
                        .P();
                if (category.length() > 0) {
                    this
                            .CODE()
                            .escape('[')
                            .escape(category)
                            .escape(']')
                            ._CODE()
                            .space();
                }
                this
                        .localize(entry.getMessage())
                        ._P()
                        ._LI();
            }
            if (withMax) {
                this
                        .LI()
                        .P()
                        .localize(LocalisationUtils.toMessage("_ info.admin.moreerrors", (messageCount - 5)))
                        ._P()
                        ._LI();
            }
            this
                    ._UL();
            this
                    ._LI();
        }
        this
                ._UL();

    }


    private void printConfState() {
        startBSRow();
        this
                .H2()
                .localize("_ title.sctconf.files")
                ._H2();
        startPanel();
        short state = confState.getGlobalState();
        boolean printConfDetails = false;
        switch (state) {
            case ConfState.GLOBAL_CONFIO_ERROR:
                this
                        .P()
                        .SPAN(ERROR_ATTR)
                        .localize("_ error.global.conf_io")
                        .escape(" : ")
                        .escape(confState.getConfIOMessage())
                        ._SPAN()
                        ._P();
                break;
            case ConfState.GLOBAL_UNDEFINED:
                this
                        .P()
                        .SPAN(ERROR_ATTR)
                        .localize("_ error.global.conf_null")
                        .escape(" : ")
                        .escape(confState.getConfIOMessage())
                        ._SPAN()
                        ._P();
                break;
            case ConfState.GLOBAL_OK:
                this
                        .UL();
                printConfFileLi(ConfConstants.SOURCES_NAME);
                printConfFileLi(ConfConstants.METADATA_NAME);
                printConfFileLi(ConfConstants.ATTRIBUTES_NAME);
                printConfFileLi(ConfConstants.CATEGORIES_NAME);
                printConfFileLi(ConfConstants.URI_CODES_NAME);
                printConfFileLi(ConfConstants.OPTIONS_NAME);
                printConfFileLi(ConfConstants.VALIDATION_NAME);
                printConfFileLi(ConfConstants.FIELDS_NAME);
                this
                        ._UL();
                printConfDetails = true;
                break;
        }
        CommandBox commandBox = CB.action(".")
                .family("SCT")
                .name(ConfUpdateCommand.COMMANDNAME)
                .lockey(ConfUpdateCommand.COMMANDKEY);
        startCommandBox(commandBox);
        endCommandBox("_ submit.admin.confupdate");
        endPanel();
        endBSRow();
        if (printConfDetails) {
            startBSRow();
            this
                    .H2()
                    .localize("_ title.sctconf.details")
                    ._H2();
            startPanel();
            printSourcesDetails();
            printMetadataDetails();
            printAttributesDetails();
            printCategoriesDetails();
            printUriCodesDetails();
            printOptionsDetails();
            printValidationDetails();
            printFieldsDetails();
            endPanel();
            endBSRow();
        }

    }

    private void printConfFileLi(String name) {
        this
                .LI()
                .P();
        printFileInfo(name, true);
        this
                ._P()
                ._LI();
    }

    private void printSourcesDetails() {
        this
                .H3(HA.id(ConfConstants.SOURCES_NAME))
                .localize("_ title.sctconf.sources")
                .escape(" = ");
        printFileInfo(ConfConstants.SOURCES_NAME, false);
        this
                ._H3();
        startPanel();
        printConfLog(ConfConstants.SOURCES_NAME);
        SourcesPrint.printScrutariSourceList(this);
        endPanel();
    }

    private void printMetadataDetails() {
        this
                .H3(HA.id(ConfConstants.METADATA_NAME))
                .localize("_ title.sctconf.metadata")
                .escape(" = ");
        printFileInfo(ConfConstants.METADATA_NAME, false);
        this
                ._H3();
        startPanel();
        printConfLog(ConfConstants.METADATA_NAME);
        MetadataPrint.printMetadata(this, engine.getEngineMetadata());
        endPanel();
    }

    private void printAttributesDetails() {
        this
                .H3(HA.id(ConfConstants.ATTRIBUTES_NAME))
                .localize("_ title.sctconf.attributes")
                .escape(" = ");
        printFileInfo(ConfConstants.ATTRIBUTES_NAME, false);
        this
                ._H3();
        startPanel();
        printConfLog(ConfConstants.ATTRIBUTES_NAME);
        AttributeDefManager attributeDefManager = engine.getAttributeDefManager();
        if (attributeDefManager.getGroupNameSet().isEmpty()) {
            this
                    .P()
                    .SPAN(HA.classes("scrutari-admin-Info"))
                    .localize("_ info.sctconf.none_attributes")
                    ._SPAN()
                    ._P();
        } else {
            printAttributesList(attributeDefManager.getAttributeDefList(AttributeDefManager.PRIMARY_GROUP), "_ title.sctconf.attributes_primary");
            printAttributesList(attributeDefManager.getAttributeDefList(AttributeDefManager.SECONDARY_GROUP), "_ title.sctconf.attributes_secondary");
            printAttributesList(attributeDefManager.getAttributeDefList(AttributeDefManager.TECHNICAL_GROUP), "_ title.sctconf.attributes_technical");
        }
        endPanel();
    }

    private void printAttributesList(List<AttributeDef> attributeDefList, String titleMessageKey) {
        if (attributeDefList.isEmpty()) {
            return;
        }
        this
                .H4()
                .localize(titleMessageKey)
                ._H4();
        startPanel();
        AttributesPrint.printAttributesList(this, attributeDefList);
        endPanel();
    }


    private void printCategoriesDetails() {
        this
                .H3(HA.id(ConfConstants.CATEGORIES_NAME))
                .localize("_ title.sctconf.categories")
                .escape(" = ");
        printFileInfo(ConfConstants.CATEGORIES_NAME, false);
        this
                ._H3();
        startPanel();
        printConfLog(ConfConstants.CATEGORIES_NAME);
        CategoriesPrint.printCategories(this);
        endPanel();
    }

    private void printUriCodesDetails() {
        this
                .H3(HA.id(ConfConstants.URI_CODES_NAME))
                .localize("_ title.sctconf.uricodes")
                .escape(" = ");
        printFileInfo(ConfConstants.URI_CODES_NAME, false);
        this
                ._H3();
        startPanel();
        printConfLog(ConfConstants.URI_CODES_NAME);
        UriCodesPrint.printUriCodes(this);
        endPanel();
    }


    private void printOptionsDetails() {
        this
                .H3(HA.id(ConfConstants.OPTIONS_NAME))
                .localize("_ title.sctconf.options")
                .escape(" = ");
        printFileInfo(ConfConstants.OPTIONS_NAME, false);
        this
                ._H3();
        startPanel();
        printConfLog(ConfConstants.OPTIONS_NAME);
        this
                .H4()
                .localize("_ title.sctconf.options_ponderation")
                ._H4();
        GlobalSearchOptions globalSearchOptions = engine.getScrutariSession().getGlobalSearchOptions();
        PertinencePonderation pertinencePonderation = globalSearchOptions.getPertinencePonderation();
        startPanel();
        this
                .TABLE();
        printPonderatioRow("_ label.sctconf.ponderation_occurrence", pertinencePonderation.getOccurrencePonderation());
        printPonderatioRow("_ label.sctconf.ponderation_date", pertinencePonderation.getDatePonderation());
        printPonderatioRow("_ label.sctconf.ponderation_origin", pertinencePonderation.getOriginPonderation());
        this
                ._TABLE();
        endPanel();
        endPanel();
    }

    private void printValidationDetails() {
        this
                .H3(HA.id(ConfConstants.VALIDATION_NAME))
                .localize("_ title.sctconf.validation")
                .escape(" = ");
        printFileInfo(ConfConstants.VALIDATION_NAME, false);
        this
                ._H3();
        startPanel();
        printConfLog(ConfConstants.VALIDATION_NAME);
        ValidationPrint.printValidator(this);
        endPanel();
    }

    private void printFieldsDetails() {
        this
                .H3(HA.id(ConfConstants.FIELDS_NAME))
                .localize("_ title.sctconf.fields")
                .escape(" = ");
        printFileInfo(ConfConstants.FIELDS_NAME, false);
        this
                ._H3();
        startPanel();
        printConfLog(ConfConstants.FIELDS_NAME);
        FieldsPrint.printFieldVariantList(this);
        endPanel();
    }

    private void printPonderatioRow(String key, float f) {
        this
                .TR()
                .TD()
                .localize(key)
                .colon()
                .nonBreakableSpace()
                ._TD()
                .TD()
                .escape(String.valueOf(Math.round(f * 100)))
                .escape('%')
                ._TD()
                ._TR();
    }

    private void printRestartForm() {
        CommandBox commandBox = CB.action(".")
                .family("SCT")
                .name(RestartCommand.COMMANDNAME)
                .lockey(RestartCommand.COMMANDKEY);
        startCommandBox(commandBox);
        this
                .P()
                .LABEL()
                .INPUT_checkbox(RestartCommand.FORCE_PARAMNAME, "1", false)
                .space()
                .localize("_ label.admin.forcerestart")
                ._LABEL()
                ._P();
        endCommandBox("_ submit.admin.restart");
    }

    private void printConfLog(String name) {
        short fileState = confState.getFileState(name);
        switch (fileState) {
            case ConfConstants.FILE_STATE_UNKNOWN:
            case ConfConstants.FILE_MISSING:
            case ConfConstants.FILE_OK:
                return;
        }
        String logContent = engine.getEngineStorage().getLogStorage().getLog(SctLogConstants.CONF_LOG, name + ".xml");
        if (logContent == null) {
            return;
        }
        List<CategorizedMessageEntry> messageList = LogUtils.readCategorizedMessageList(logContent);
        if (!messageList.isEmpty()) {
            this
                    .H4()
                    .localize("_ title.admin.log")
                    ._H4();
            this
                    .UL();
            for (CategorizedMessageEntry entry : messageList) {
                String category = entry.getCategory();
                this
                        .LI()
                        .P();
                if (category.length() > 0) {
                    this
                            .CODE()
                            .escape('[')
                            .escape(category)
                            .escape(']')
                            ._CODE()
                            .space();
                }
                this
                        .localize(entry.getMessage())
                        ._P()
                        ._LI();
            }
            this
                    ._UL();
        }
    }

    private void printFileInfo(String name, boolean withLink) {
        short fileState = confState.getFileState(name);
        if (withLink) {
            this
                    .A(HA.href("#" + name));
        }
        this
                .escape(name)
                .escape(".xml");
        if (withLink) {
            this
                    ._A();
        }
        this
                .space()
                .escape('(')
                .SPAN(HA.classes(getSpanClass(fileState)))
                .localize(getFileStateMessageKey(fileState))
                ._SPAN();
        escape(')');
    }

    private static String getSpanClass(short fileState) {
        switch (fileState) {
            case ConfConstants.FILE_STATE_UNKNOWN:
                return "scrutari-admin-State_Neutral";
            case ConfConstants.FILE_MISSING:
                return "scrutari-admin-State_Neutral";
            case ConfConstants.FILE_XML_MALFORMED:
                return "scrutari-admin-State_Error";
            case ConfConstants.FILE_XML_ERROR:
                return "scrutari-admin-State_Error";
            case ConfConstants.FILE_XML_WARNING:
                return "scrutari-admin-State_Warning";
            case ConfConstants.FILE_OK:
                return "scrutari-admin-State_Ok";
            default:
                throw new SwitchException("Unknown getFileState : " + fileState);
        }
    }

    private static String getFileStateMessageKey(short fileState) {
        switch (fileState) {
            case ConfConstants.FILE_STATE_UNKNOWN:
                return "_ state.file.unknownstate";
            case ConfConstants.FILE_MISSING:
                return "_ state.file.missing";
            case ConfConstants.FILE_XML_MALFORMED:
                return "_ state.file.xml_malformed";
            case ConfConstants.FILE_XML_ERROR:
                return "_ state.file.xml_error";
            case ConfConstants.FILE_XML_WARNING:
                return "_ state.file.xml_warning";
            case ConfConstants.FILE_OK:
                return "_ state.file.ok";
            default:
                throw new SwitchException("Unknown getFileState : " + fileState);
        }
    }

}
