/* Scrutari - Copyright (c) 2012-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.export.contentstreams;

import fr.exemole.sctserver.api.ScrutariSession;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.opendocument.io.OdSource;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdXML;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.opendocument.io.OdZipEngine;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.mimetype.MimeTypeConstants;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.scrutari.data.BaseData;
import net.scrutari.data.DataConstants;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.codes.Codes;
import net.scrutari.db.api.codes.IndexationCodes;
import net.scrutari.db.tools.util.BaseCheck;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.CategoryDef;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public class ResultOdsStream implements ResponseHandler {

    private final ScrutariSession scrutariSession;
    private final FicheSearchResult ficheSearchResult;
    private final Lang lang;

    public ResultOdsStream(ScrutariSession scrutariSession, FicheSearchResult ficheSearchResult, Lang lang) {
        this.scrutariSession = scrutariSession;
        this.ficheSearchResult = ficheSearchResult;
        this.lang = lang;
    }

    @Override
    public long getLastModified() {
        return ResponseHandler.LASTMODIFIED_NOCACHE;
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(MimeTypeConstants.ODS);
        OutputStream outputStream = response.getOutputStream();
        OdZip odZip = OdZip.newInstance(OdUtils.OD_SPREADSHEET).setContentOdSource(new ContentOdSource());
        OdZipEngine.run(outputStream, odZip);
        outputStream.close();
    }


    private class ContentOdSource implements OdSource {

        private ContentOdSource() {
        }

        @Override
        public void writeStream(OutputStream outputStream) throws IOException {
            BufferedWriter buf = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
            AppendableXMLWriter xmlWriter = XMLUtils.toXMLWriter(buf);
            xmlWriter.appendXMLDeclaration();
            ResultXMLPart resultXMLPart = new ResultXMLPart(xmlWriter);
            resultXMLPart.start();
            resultXMLPart.addResult();
            resultXMLPart.end();
            buf.flush();
        }


        private class ResultXMLPart extends XMLPart {

            private final ScrutariDB scrutariDB;
            private final DataAccess dataAccess;
            private final GlobalSearchOptions globalSearchOptions;
            private final Map<Integer, BaseInfo> baseInfoMap = new HashMap<Integer, BaseInfo>();
            private final Map<Integer, String> motcleLibMap = new HashMap<Integer, String>();

            private ResultXMLPart(XMLWriter xmlWriter) {
                super(xmlWriter);
                this.scrutariDB = scrutariSession.getScrutariDB();
                this.dataAccess = scrutariDB.openDataAccess();
                this.globalSearchOptions = scrutariSession.getGlobalSearchOptions();
            }

            private void start() throws IOException {
                OdXML.openDocumentContent(this);
                OdXML.openBody(this);
                OdXML.openSpreadsheet(this);
            }

            private void end() throws IOException {
                OdXML.closeSpreadsheet(this);
                OdXML.closeBody(this);
                OdXML.closeDocumentContent(this);
                dataAccess.close();
            }

            private void addResult() throws IOException {
                List<FicheSearchResultGroup> groupList = ficheSearchResult.getFicheSearchResultGroupList();
                int groupLength = groupList.size();
                for (int i = 0; i < groupLength; i++) {
                    FicheSearchResultGroup ficheSearchResultGroup = groupList.get(i);
                    List<FicheSearchResultInfo> infoList = ficheSearchResultGroup.getFicheSearchResultInfoList();
                    int ficheByGroupLength = infoList.size();
                    String tableName;
                    Category category = globalSearchOptions.getCategoryByRank(ficheSearchResultGroup.getCategoryRank());
                    if (category != null) {
                        CategoryDef categoryDef = category.getCategoryDef();
                        tableName = LabelUtils.seekLabelString(categoryDef, lang, categoryDef.getName());
                    } else {
                        if (groupLength == 1) {
                            tableName = "Result";
                        } else {
                            int pos = i + 1;
                            tableName = "Result " + pos;
                        }
                    }
                    OdXML.openTable(this, tableName);
                    for (int j = 0; j < ficheByGroupLength; j++) {
                        FicheSearchResultInfo ficheSearchResultInfo = infoList.get(j);
                        int codefiche = ficheSearchResultInfo.getCode();
                        FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
                        addFicheData(ficheData);
                    }
                    OdXML.closeTable(this);

                }
            }

            private void addFicheData(FicheData ficheData) throws IOException {
                int codefiche = ficheData.getCodeFiche();
                BaseInfo baseInfo = getBaseInfo(ficheData.getCodeBase());
                OdXML.openTableRow(this);
                OdXML.addStringTableCell(this, ficheData.getTitre());
                OdXML.addStringTableCell(this, ficheData.getSoustitre());
                FuzzyDate date = ficheData.getDate();
                if (date != null) {
                    OdXML.addNumberTableCell(this, date.getYear());
                } else {
                    OdXML.addEmptyTableCell(this);
                }
                OdXML.addStringTableCell(this, baseInfo.labelString);
                OdXML.addStringTableCell(this, ficheData.getLang().toString());
                OdXML.addLinkStringTableCell(this, ficheData.getHref(baseInfo.baseCheck.getCheckedLang()));
                int max = ficheData.getComplementMaxNumber();
                StringBuilder motcleBuilder = new StringBuilder();
                Codes thesaurusCodes = dataAccess.getThesaurusCodes();
                int thesaurusCount = thesaurusCodes.getCodeCount();
                for (int i = 0; i < thesaurusCount; i++) {
                    IndexationCodes indexationCodes = dataAccess.getIndexationCodes(codefiche, thesaurusCodes.getCode(i));
                    int indexationCount = indexationCodes.getCodeCount();
                    for (int j = 0; j < indexationCount; j++) {
                        int codemotcle = indexationCodes.getCode(j);
                        String libelleString = getMotcleLib(codemotcle);
                        if (libelleString.length() > 0) {
                            if (motcleBuilder.length() > 0) {
                                motcleBuilder.append("; ");
                            }
                            motcleBuilder.append(libelleString);
                        }
                    }
                }
                OdXML.addStringTableCell(this, motcleBuilder.toString());
                for (int i = 1; i <= max; i++) {
                    OdXML.addStringTableCell(this, ficheData.getComplementByNumber(i));
                }
                OdXML.closeTableRow(this);
            }

            private String getMotcleLib(int codemotcle) {
                String libelleString = motcleLibMap.get(codemotcle);
                if (libelleString != null) {
                    return libelleString;
                }
                MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                Label libelle = motcleData.getLabelByLang(lang);
                if (libelle == null) {
                    if (motcleData.getLabelCount() > 0) {
                        libelle = motcleData.getLabelByIndex(0);
                    }
                }
                if (libelle != null) {
                    libelleString = libelle.getLabelString();
                } else {
                    libelleString = "";
                }
                motcleLibMap.put(codemotcle, libelleString);
                return libelleString;
            }

            private BaseInfo getBaseInfo(int codebase) {
                BaseInfo baseCheck = baseInfoMap.get(codebase);
                if (baseCheck != null) {
                    return baseCheck;
                }
                BaseData baseData = dataAccess.getBaseDataByCode(codebase);
                baseCheck = new BaseInfo(baseData);
                baseInfoMap.put(codebase, baseCheck);
                return baseCheck;
            }


            private class BaseInfo {

                private final BaseData baseData;
                private final BaseCheck baseCheck;
                private final String labelString;

                private BaseInfo(BaseData baseData) {
                    this.baseData = baseData;
                    this.baseCheck = BaseCheck.check(baseData, lang);
                    this.labelString = LabelUtils.seekLabelString(baseData.getPhrases(), DataConstants.BASE_TITLE, baseCheck.getCheckedLang(), baseData.getBaseURI().getBaseName());
                }

            }

        }

    }

}
