/* ScrutariLib_Data - Copyright (c) 2005-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import java.io.IOException;
import java.io.Serializable;
import java.net.URI;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.util.attr.AppendableAttributeHolderBuilder;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.Phrases;
import net.mapeadores.util.text.PhrasesBuilder;
import net.scrutari.datauri.BaseURI;


/**
 *
 * @author Vincent Calame
 */
public final class BaseData implements Serializable {

    private final BaseURI baseURI;
    private final int codebase;
    private final String scrutariSourceName;
    private final Lang[] availableLangArray;
    private final String baseIconPart1;
    private final String baseIconPart2;
    private final Phrases phrases;
    private final AttributeHolder attributeHolder;
    private final int[] codesCorpus;
    private final int[] codesThesaurus;


    private BaseData(BaseURI baseURI, int codebase, String scrutariSourceName, Lang[] availableLangArray, String baseIconPart1, String baseIconPart2, Phrases phrases, AttributeHolder attributeHolder, int[] codesCorpus, int[] codesThesaurus) {
        this.baseURI = baseURI;
        this.codebase = codebase;
        this.scrutariSourceName = scrutariSourceName;
        this.availableLangArray = availableLangArray;
        this.baseIconPart1 = baseIconPart1;
        this.baseIconPart2 = baseIconPart2;
        this.phrases = phrases;
        this.attributeHolder = attributeHolder;
        this.codesCorpus = codesCorpus;
        this.codesThesaurus = codesThesaurus;
    }

    public String getScrutariSourceName() {
        return scrutariSourceName;
    }

    public BaseURI getBaseURI() {
        return baseURI;
    }

    public int getCodeBase() {
        return codebase;
    }

    public int getAvailableLangCount() {
        return availableLangArray.length;
    }

    public Lang getAvailableLang(int i) {
        return availableLangArray[i];
    }

    public Lang checkAvailableLang(Lang lang) {
        int length = availableLangArray.length;
        if (length == 0) {
            return lang;
        }
        for (int i = 0; i < length; i++) {
            if (lang.equals(availableLangArray[i])) {
                return lang;
            }
        }
        if (!lang.isRootLang()) {
            Lang rootLang = lang.getRootLang();
            for (int i = 0; i < length; i++) {
                if (rootLang.equals(availableLangArray[i])) {
                    return rootLang;
                }
            }
        }
        return availableLangArray[0];
    }

    public String getBaseIcon(Lang lang) {
        if (baseIconPart2 == null) {
            return baseIconPart1;
        }
        StringBuilder buf = new StringBuilder(baseIconPart1.length() + baseIconPart2.length() + 10);
        buf.append(baseIconPart1);
        buf.append(lang.toString());
        buf.append(baseIconPart2);
        return buf.toString();
    }

    public int getCorpusCount() {
        return codesCorpus.length;
    }

    public int getCodeCorpus(int i) {
        return codesCorpus[i];
    }

    public int getThesaurusCount() {
        return codesThesaurus.length;
    }

    public int getCodeThesaurus(int i) {
        return codesThesaurus[i];
    }

    public Phrases getPhrases() {
        return phrases;
    }

    public AttributeHolder getAttributHolder() {
        return attributeHolder;
    }

    public static BaseData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        BaseURI baseURI = (BaseURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
        int codebase = primitivesReader.readInt();
        String scrutariSourceName = primitivesReader.readString();
        int langsUILength = primitivesReader.readInt();
        Lang[] langsUI = new Lang[langsUILength];
        for (int i = 0; i < langsUILength; i++) {
            langsUI[i] = Lang.build(primitivesReader.readString());
        }
        String baseIconPart1 = primitivesReader.readString();
        String baseIconPart2 = primitivesReader.readString();
        Phrases phrases = DataPhraseUtils.fromPrimitives(primitivesReader);
        AttributeHolder attributeHolder = ScrutariAttributes.fromPrimitives(primitivesReader);
        int corpusLength = primitivesReader.readInt();
        int[] codesCorpus = new int[corpusLength];
        for (int i = 0; i < corpusLength; i++) {
            codesCorpus[i] = primitivesReader.readInt();
        }
        int thesaurusLength = primitivesReader.readInt();
        int[] codesThesaurus = new int[thesaurusLength];
        for (int i = 0; i < thesaurusLength; i++) {
            codesThesaurus[i] = primitivesReader.readInt();
        }
        return new BaseData(baseURI, codebase, scrutariSourceName, langsUI, baseIconPart1, baseIconPart2, phrases, attributeHolder, codesCorpus, codesThesaurus);
    }

    public static void toPrimitives(BaseData baseData, PrimitivesWriter primitivesWriter) throws IOException {
        URIPrimitivesUtils.writeScrutariDataURI(baseData.baseURI, primitivesWriter);
        primitivesWriter.writeInt(baseData.codebase);
        primitivesWriter.writeString(baseData.scrutariSourceName);
        int availableLangLength = baseData.availableLangArray.length;
        primitivesWriter.writeInt(availableLangLength);
        for (int i = 0; i < availableLangLength; i++) {
            primitivesWriter.writeString(baseData.availableLangArray[i].toString());
        }
        primitivesWriter.writeString(baseData.baseIconPart1);
        primitivesWriter.writeString(baseData.baseIconPart2);
        DataPhraseUtils.toPrimitives(baseData.phrases, primitivesWriter);
        ScrutariAttributes.toPrimitives(baseData.attributeHolder, primitivesWriter);
        int corpusLength = baseData.codesCorpus.length;
        primitivesWriter.writeInt(corpusLength);
        for (int i = 0; i < corpusLength; i++) {
            primitivesWriter.writeInt(baseData.codesCorpus[i]);
        }
        int thesaurusLength = baseData.codesThesaurus.length;
        primitivesWriter.writeInt(thesaurusLength);
        for (int i = 0; i < thesaurusLength; i++) {
            primitivesWriter.writeInt(baseData.codesThesaurus[i]);
        }
    }


    public static class Builder {

        private final BaseURI baseURI;
        private final List<Integer> corpusList = new ArrayList<Integer>();
        private final List<Integer> thesaurusList = new ArrayList<Integer>();
        private final Set<Lang> availableLangSet = new LinkedHashSet<Lang>();
        private final PhrasesBuilder phrasesBuilder = new PhrasesBuilder();
        private final AppendableAttributeHolderBuilder attributeHolderBuilder = new AppendableAttributeHolderBuilder();
        private String baseIcon;

        public Builder(BaseURI baseURI) {
            this.baseURI = baseURI;
        }

        public void addLabel(String name, Label label) {
            label = DataPhraseUtils.checkLabel(label);
            if (label == null) {
                return;
            }
            phrasesBuilder.getPhraseBuilder(name).putLabel(label);
        }

        public void addAttribute(AttributeKey attributeKey, List<CleanedString> valueList) {
            ScrutariAttributes.addAttribute(attributeHolderBuilder, attributeKey, valueList);
        }

        public void addAvailableLang(Lang lang) {
            availableLangSet.add(lang);
        }

        public BaseURI getBaseURI() {
            return baseURI;
        }

        public boolean hasBaseIcon() {
            return (this.baseIcon != null);
        }

        public boolean setBaseIcon(URI baseIconURI) {
            if (baseIconURI == null) {
                this.baseIcon = null;
                return true;
            }
            if (!baseIconURI.isAbsolute()) {
                return false;
            }
            if (baseIconURI.isOpaque()) {
                return false;
            }
            String scheme = baseIconURI.getScheme();
            if ((!scheme.equals("http")) && (!scheme.equals("https"))) {
                return false;
            }
            this.baseIcon = baseIconURI.toASCIIString();
            return true;
        }

        public void addCodeCorpus(int codecorpus) {
            corpusList.add(codecorpus);
        }

        public void addCodeThesaurus(int codethesaurus) {
            thesaurusList.add(codethesaurus);
        }

        public BaseData toBaseData(int codebase, String scrutariSourceName) {
            int corpusCount = corpusList.size();
            int[] codesCorpus = new int[corpusCount];
            for (int i = 0; i < corpusCount; i++) {
                codesCorpus[i] = corpusList.get(i);
            }
            int thesaurusCount = thesaurusList.size();
            int[] codesThesaurus = new int[thesaurusCount];
            for (int i = 0; i < thesaurusCount; i++) {
                codesThesaurus[i] = thesaurusList.get(i);
            }
            Lang[] availableLangArray = availableLangSet.toArray(new Lang[availableLangSet.size()]);
            String baseIconPart1 = null;
            String baseIconPart2 = null;
            if (baseIcon != null) {
                int idx = baseIcon.indexOf("$LANGUI");
                if (idx == -1) {
                    baseIconPart1 = baseIcon;
                } else {
                    baseIconPart1 = baseIcon.substring(0, idx);
                    baseIconPart2 = baseIcon.substring(idx + 7);
                }
            }
            Phrases phrases = phrasesBuilder.toPhrases();
            AttributeHolder attributeHolder = attributeHolderBuilder.toAttributeHolder();
            return new BaseData(baseURI, codebase, scrutariSourceName, availableLangArray, baseIconPart1, baseIconPart2, phrases, attributeHolder, codesCorpus, codesThesaurus);
        }

    }

}
