/* ScrutariLib_Data - Copyright (c) 2005-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import java.io.IOException;
import java.io.Serializable;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.attr.AppendableAttributeHolderBuilder;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.Phrases;
import net.mapeadores.util.text.PhrasesBuilder;
import net.scrutari.datauri.CorpusURI;


/**
 *
 * @author Vincent Calame
 */
public final class CorpusData implements Serializable {

    private final CorpusURI corpusURI;
    private final int codecorpus;
    private final int codebase;
    private final int complementCount;
    private final Phrases phrases;
    private final AttributeHolder attributeHolder;
    private final int[] codesFiches;

    private CorpusData(CorpusURI corpusURI, int codecorpus, int codebase, int complementCount, Phrases phrases, AttributeHolder attributeHolder, int[] codesFiches) {
        this.corpusURI = corpusURI;
        this.codecorpus = codecorpus;
        this.codebase = codebase;
        this.complementCount = complementCount;
        this.phrases = phrases;
        this.attributeHolder = attributeHolder;
        this.codesFiches = codesFiches;
    }

    public CorpusURI getCorpusURI() {
        return corpusURI;
    }

    /**
     * Retourne le nombre maximum de champs complémentaires.
     */
    public int getComplementMaxNumber() {
        return complementCount;
    }

    /**
     * Retourne le code du corpus.
     */
    public int getCodeCorpus() {
        return codecorpus;
    }

    /**
     * Retourne le code de la base.
     */
    public int getCodeBase() {
        return codebase;
    }

    public Phrases getPhrases() {
        return phrases;
    }

    public AttributeHolder getAttributHolder() {
        return attributeHolder;
    }

    /**
     * Retourne le total de fiches du corpus.
     */
    public int getFicheCount() {
        return codesFiches.length;
    }

    /**
     * Retourne le code de la i-ième fiche du corpus.
     */
    public int getCodeFiche(int i) {
        return codesFiches[i];
    }


    public static CorpusData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        CorpusURI corpusURI = (CorpusURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
        int codecorpus = primitivesReader.readInt();
        int codebase = primitivesReader.readInt();
        int complementCount = primitivesReader.readInt();
        Phrases phrases = DataPhraseUtils.fromPrimitives(primitivesReader);
        AttributeHolder attributeHolder = ScrutariAttributes.fromPrimitives(primitivesReader);
        int fichesLength = primitivesReader.readInt();
        int[] codesFiches = new int[fichesLength];
        for (int i = 0; i < fichesLength; i++) {
            codesFiches[i] = primitivesReader.readInt();
        }
        return new CorpusData(corpusURI, codecorpus, codebase, complementCount, phrases, attributeHolder, codesFiches);
    }

    public static void toPrimitives(CorpusData corpusData, PrimitivesWriter primitivesWriter) throws IOException {
        URIPrimitivesUtils.writeScrutariDataURI(corpusData.corpusURI, primitivesWriter);
        primitivesWriter.writeInt(corpusData.codecorpus);
        primitivesWriter.writeInt(corpusData.codebase);
        int complementCount = corpusData.complementCount;
        primitivesWriter.writeInt(complementCount);
        DataPhraseUtils.toPrimitives(corpusData.phrases, primitivesWriter);
        ScrutariAttributes.toPrimitives(corpusData.attributeHolder, primitivesWriter);
        int fichesLength = corpusData.codesFiches.length;
        primitivesWriter.writeInt(fichesLength);
        for (int i = 0; i < fichesLength; i++) {
            primitivesWriter.writeInt(corpusData.codesFiches[i]);
        }
    }


    public static class Builder {

        private final CorpusURI corpusURI;
        private final PhrasesBuilder phrasesBuilder = new PhrasesBuilder();
        private final AppendableAttributeHolderBuilder attributeHolderBuilder = new AppendableAttributeHolderBuilder();
        private final List<Integer> codeficheList = new ArrayList<Integer>();
        private String hrefParent;
        private int complementCount = 0;

        public Builder(CorpusURI corpusURI) {
            this.corpusURI = corpusURI;
        }

        public CorpusURI getCorpusURI() {
            return corpusURI;
        }

        public void addLabel(String name, Label label) {
            label = DataPhraseUtils.checkLabel(label);
            if (label == null) {
                return;
            }
            phrasesBuilder.getPhraseBuilder(name).putLabel(label);
        }

        public void addAttribute(AttributeKey attributeKey, List<CleanedString> valueList) {
            ScrutariAttributes.addAttribute(attributeHolderBuilder, attributeKey, valueList);
        }

        public int createNewComplement() {
            complementCount++;
            return complementCount;
        }

        public int getComplementMaxNumber() {
            return complementCount;
        }

        public void addCodeFiche(int codefiche) {
            codeficheList.add(codefiche);
        }

        public CorpusData toCorpusData(int codecorpus, int codebase) {
            int ficheCount = codeficheList.size();
            int[] codesFiches = new int[ficheCount];
            for (int i = 0; i < ficheCount; i++) {
                codesFiches[i] = codeficheList.get(i);
            }
            Phrases phrases = phrasesBuilder.toPhrases();
            AttributeHolder attributeHolder = attributeHolderBuilder.toAttributeHolder();
            return new CorpusData(corpusURI, codecorpus, codebase, complementCount, phrases, attributeHolder, codesFiches);
        }

        public String getHrefParent() {
            return hrefParent;
        }

        public boolean setHrefParent(URI hrefParentURI) {
            if (hrefParentURI == null) {
                this.hrefParent = null;
                return true;
            }
            if (!hrefParentURI.isAbsolute()) {
                return false;
            }
            if (hrefParentURI.isOpaque()) {
                return false;
            }
            String scheme = hrefParentURI.getScheme();
            if ((!scheme.equals("http")) && (!scheme.equals("https"))) {
                return false;
            }
            this.hrefParent = hrefParentURI.toASCIIString();
            return true;
        }

    }

}
