/* ScrutariLib_Data - Copyright (c) 2005-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.attr.AppendableAttributeHolderBuilder;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.Phrases;
import net.mapeadores.util.text.PhrasesBuilder;
import net.scrutari.datauri.ThesaurusURI;


/**
 *
 * @author Vincent Calame
 */
public final class ThesaurusData implements Serializable {

    private final ThesaurusURI thesaurusURI;
    private final int codethesaurus;
    private final int codebase;
    private final Phrases phrases;
    private final AttributeHolder attributeHolder;
    private final int[] codesMotscles;

    private ThesaurusData(ThesaurusURI thesaurusURI, int codethesaurus, int codebase, Phrases phrases, AttributeHolder attributeHolder, int[] codesMotscles) {
        this.thesaurusURI = thesaurusURI;
        this.codethesaurus = codethesaurus;
        this.codebase = codebase;
        this.phrases = phrases;
        this.attributeHolder = attributeHolder;
        this.codesMotscles = codesMotscles;
    }

    public ThesaurusURI getThesaurusURI() {
        return thesaurusURI;
    }

    public int getCodeThesaurus() {
        return codethesaurus;
    }

    public int getCodeBase() {
        return codebase;
    }

    public Phrases getPhrases() {
        return phrases;
    }

    public AttributeHolder getAttributHolder() {
        return attributeHolder;
    }

    public int getMotcleCount() {
        return codesMotscles.length;
    }

    public int getCodeMotcle(int i) {
        return codesMotscles[i];
    }

    public static ThesaurusData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        ThesaurusURI thesaurusURI = (ThesaurusURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
        int codethesaurus = primitivesReader.readInt();
        int codebase = primitivesReader.readInt();
        Phrases phrases = DataPhraseUtils.fromPrimitives(primitivesReader);
        AttributeHolder attributeHolder = ScrutariAttributes.fromPrimitives(primitivesReader);
        int motsclesLength = primitivesReader.readInt();
        int[] codesMotscles = new int[motsclesLength];
        for (int i = 0; i < motsclesLength; i++) {
            codesMotscles[i] = primitivesReader.readInt();
        }
        return new ThesaurusData(thesaurusURI, codethesaurus, codebase, phrases, attributeHolder, codesMotscles);
    }

    public static void toPrimitives(ThesaurusData thesaurusData, PrimitivesWriter primitivesWriter) throws IOException {
        URIPrimitivesUtils.writeScrutariDataURI(thesaurusData.thesaurusURI, primitivesWriter);
        primitivesWriter.writeInt(thesaurusData.codethesaurus);
        primitivesWriter.writeInt(thesaurusData.codebase);
        DataPhraseUtils.toPrimitives(thesaurusData.phrases, primitivesWriter);
        ScrutariAttributes.toPrimitives(thesaurusData.attributeHolder, primitivesWriter);
        int motsclesLength = thesaurusData.codesMotscles.length;
        primitivesWriter.writeInt(motsclesLength);
        for (int i = 0; i < motsclesLength; i++) {
            primitivesWriter.writeInt(thesaurusData.codesMotscles[i]);
        }
    }


    public static class Builder {

        private final ThesaurusURI thesaurusURI;
        private final PhrasesBuilder phrasesBuilder = new PhrasesBuilder();
        private final AppendableAttributeHolderBuilder attributeHolderBuilder = new AppendableAttributeHolderBuilder();
        private final List<Integer> motcleList = new ArrayList<Integer>();


        public Builder(ThesaurusURI thesaurusURI) {
            this.thesaurusURI = thesaurusURI;
        }

        public ThesaurusURI getThesaurusURI() {
            return thesaurusURI;
        }

        public void addLabel(String name, Label label) {
            label = DataPhraseUtils.checkLabel(label);
            if (label == null) {
                return;
            }
            phrasesBuilder.getPhraseBuilder(name).putLabel(label);
        }

        public void addAttribute(AttributeKey attributeKey, List<CleanedString> valueList) {
            ScrutariAttributes.addAttribute(attributeHolderBuilder, attributeKey, valueList);
        }

        public void addCodeMotcle(int codemotcle) {
            motcleList.add(codemotcle);
        }

        public ThesaurusData toThesaurusData(int codethesaurus, int codebase) {
            int motcleCount = motcleList.size();
            int[] codesMotscles = new int[motcleCount];
            for (int i = 0; i < motcleCount; i++) {
                codesMotscles[i] = motcleList.get(i);
            }
            Phrases phrases = phrasesBuilder.toPhrases();
            AttributeHolder attributeHolder = attributeHolderBuilder.toAttributeHolder();
            return new ThesaurusData(thesaurusURI, codethesaurus, codebase, phrases, attributeHolder, codesMotscles);
        }

    }

}
