/* ScrutariLib_SearchEngine - Copyright (c) 2013-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.dom;

import java.text.ParseException;
import java.util.List;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.attr.CheckedKeyString;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logicaloperation.OperandException;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import net.scrutari.searchengine.api.CanonicalQ;
import net.scrutari.searchengine.api.operands.eligibility.EligibilityOperand;
import net.scrutari.searchengine.tools.operands.eligibility.EligibilityOperandParser;
import net.scrutari.searchengine.tools.options.ListReductionBuilder;
import net.scrutari.searchengine.tools.options.PertinencePonderationBuilder;
import net.scrutari.searchengine.tools.options.SearchOptionsDefBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchSourceBuilder;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class FicheSearchSourceDOMReader {

    private final FicheSearchSourceBuilder ficheSearchSourceBuilder;

    public FicheSearchSourceDOMReader(FicheSearchSourceBuilder ficheSearchSourceBuilder) {
        this.ficheSearchSourceBuilder = ficheSearchSourceBuilder;
    }

    public void readFicheSearchSource(Element element_xml) {
        RootHandler rootHandler = new RootHandler();
        DOMUtils.readChildren(element_xml, rootHandler);
    }


    private class RootHandler implements ElementHandler {

        private RootHandler() {
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("q")) {
                String q = DOMUtils.readSimpleElement(element);
                ficheSearchSourceBuilder.setCanonicalQ(CanonicalQ.newInstance(q));
            } else if (tagName.equals("q-info")) {
                String q = element.getAttribute("q");
                ficheSearchSourceBuilder.setCanonicalQ(CanonicalQ.newInstance(q));
            } else if (tagName.equals("options")) {
                SearchOptionsDefBuilder searchOptionsDefBuilder = new SearchOptionsDefBuilder();
                String langString = element.getAttribute("lang");
                if (langString.length() == 0) {
                    langString = element.getAttribute("lang-ui");
                }
                try {
                    Lang lang = Lang.parse(langString);
                    searchOptionsDefBuilder.setLang(lang);
                } catch (ParseException pe) {
                }
                SearchOptionsHandler handler = new SearchOptionsHandler(searchOptionsDefBuilder);
                handler.handle(element);
                ficheSearchSourceBuilder.setSearchOptions(searchOptionsDefBuilder.toSearchOptionsDef());
            }
        }

    }


    private class SearchOptionsHandler implements ElementHandler {

        private final LangsHandler langsHandler;
        private final SearchOptionsDefBuilder searchOptionsDefBuilder;

        private SearchOptionsHandler(SearchOptionsDefBuilder searchOptionsDefBuilder) {
            this.searchOptionsDefBuilder = searchOptionsDefBuilder;
            this.langsHandler = new LangsHandler(searchOptionsDefBuilder);
        }

        private void handle(Element element) {
            DOMUtils.readChildren(element, this);
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if ((tagName.equals("lang-list")) || (tagName.equals("langs"))) {
                DOMUtils.readChildren(element, langsHandler);
            } else if (tagName.equals("fiche-list")) {
                readReductionList(ScrutariDataURI.FICHEURI_TYPE, element);
            } else if (tagName.equals("base-list")) {
                readReductionList(ScrutariDataURI.BASEURI_TYPE, element);
            } else if (tagName.equals("corpus-list")) {
                readReductionList(ScrutariDataURI.CORPUSURI_TYPE, element);
            } else if (tagName.equals("thesaurus-list")) {
                readReductionList(ScrutariDataURI.THESAURUSURI_TYPE, element);
            } else if (tagName.equals("ponderation")) {
                int occurrence = 0;
                int date = 0;
                int origin = 0;
                try {
                    occurrence = Integer.parseInt(element.getAttribute("occurrence"));
                } catch (NumberFormatException nfe) {
                }
                try {
                    date = Integer.parseInt(element.getAttribute("date"));
                } catch (NumberFormatException nfe) {
                }
                try {
                    origin = Integer.parseInt(element.getAttribute("origin"));
                } catch (NumberFormatException nfe) {
                }
                searchOptionsDefBuilder.setPertinencePonderation(PertinencePonderationBuilder.toPertinencePonderation(occurrence, date, origin));
            } else if (tagName.equals("fiche-eligibility")) {
                EligibilityOperand eligibilityOperand = toEligibilityOperand(element);
                if (eligibilityOperand != null) {
                    searchOptionsDefBuilder.addFicheEligibilityOperand(eligibilityOperand);
                }
            } else if (tagName.equals("attr")) {
                readAttrElement(searchOptionsDefBuilder, element);
            }

        }

        private EligibilityOperand toEligibilityOperand(Element element) {
            String operationString = DOMUtils.readSimpleElement(element);
            try {
                return EligibilityOperandParser.parse(operationString, null, LogUtils.NULL_MULTIMESSAGEHANDLER);
            } catch (OperandException pe) {
                return null;
            }
        }

        private void readReductionList(short type, Element element) {
            ListReductionBuilder listReductionBuilder = new ListReductionBuilder(type);
            String modeAttribute = element.getAttribute("mode");
            if (modeAttribute.equals("exclude")) {
                listReductionBuilder.setExclude(true);
            }
            DOMUtils.readChildren(element, new ListReductionHandler(listReductionBuilder));
            searchOptionsDefBuilder.putListReduction(listReductionBuilder.toListReduction());
        }

    }

    private static void readAttrElement(SearchOptionsDefBuilder searchOptionsDefBuilder, Element element) {
        String nameSpaceString = element.getAttribute("ns");
        String localKeyString = element.getAttribute("key");
        CheckedKeyString nameSpace;
        try {
            nameSpace = CheckedKeyString.parse(nameSpaceString);
        } catch (ParseException pe) {
            return;
        }
        CheckedKeyString localKey;
        try {
            localKey = CheckedKeyString.parse(localKeyString);
        } catch (ParseException pe) {
            return;
        }
        AttributeKey attributeKey = AttributeKey.build(nameSpace, localKey);
        List<CleanedString> list = AttributeUtils.readValues(element);
        if (!list.isEmpty()) {
            searchOptionsDefBuilder.getAttributesBuilder().appendValues(attributeKey, list);
        }
    }


    private static class LangsHandler implements ElementHandler {

        private final SearchOptionsDefBuilder searchOptionsDefBuilder;

        private LangsHandler(SearchOptionsDefBuilder searchOptionsDefBuilder) {
            this.searchOptionsDefBuilder = searchOptionsDefBuilder;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("lang")) {
                try {
                    Lang lang = Lang.parse(DOMUtils.readSimpleElement(element));
                    searchOptionsDefBuilder.addFilterLang(lang);
                } catch (ParseException pe) {
                }
            }
        }

    }


    private static class ListReductionHandler implements ElementHandler {

        private final ListReductionBuilder listReductionBuilder;

        private ListReductionHandler(ListReductionBuilder listReductionBuilder) {
            this.listReductionBuilder = listReductionBuilder;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("uri")) {
                try {
                    ScrutariDataURI scrutariDataURI = URIParser.parse(DOMUtils.readSimpleElement(element));
                    listReductionBuilder.add(scrutariDataURI);
                } catch (URIParseException | IllegalArgumentException e) {

                }
            }
        }

    }


}
