/* ScrutariLib_SearchEngine - Copyright (c) 2011-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result.io;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.ParseException;
import java.util.List;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.io.DataInputPrimitivesReader;
import net.mapeadores.util.primitives.io.DataOutputPrimitivesWriter;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.scrutari.data.ScrutariAttributes;
import net.scrutari.data.URIPrimitivesUtils;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.codes.Codes;
import net.scrutari.searchengine.api.CanonicalQ;
import net.scrutari.searchengine.api.QId;
import net.scrutari.searchengine.api.options.ListReduction;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.FicheSearchSource;
import net.scrutari.searchengine.api.result.InAlineaOccurrences;
import net.scrutari.searchengine.api.result.InLangOccurrences;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;
import net.scrutari.searchengine.api.result.SearchTokenOccurrence;
import net.scrutari.searchengine.tools.options.ListReductionBuilder;
import net.scrutari.searchengine.tools.options.SearchOptionsDefBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchResultBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchResultGroupBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchResultInfoBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchSourceBuilder;
import net.scrutari.searchengine.tools.result.MotcleSearchResultBuilder;
import net.scrutari.searchengine.tools.result.ResultUtils;
import net.scrutari.searchengine.tools.result.SearchTokenOccurrenceBuilder;


/**
 *
 * @author Vincent Calame
 */
public final class FicheSearchResultStorage {

    private final static int CACHE_VERSION = 2;

    private FicheSearchResultStorage() {
    }

    public static void write(File file, FicheSearchResult ficheSearchResult) throws IOException {
        DataOutputStream dataOutput = new DataOutputStream(new FileOutputStream(file));
        DataOutputPrimitivesWriter primWriter = new DataOutputPrimitivesWriter(dataOutput);
        FicheSearchResultStorage.write(primWriter, ficheSearchResult);
        dataOutput.close();
    }

    public static void write(PrimitivesWriter primitivesWriter, FicheSearchResult ficheSearchResult) throws IOException {
        primitivesWriter.writeInt(CACHE_VERSION);
        writeFicheSearchSource(primitivesWriter, ficheSearchResult.getFicheSearchSource());
        primitivesWriter.writeInt(ficheSearchResult.getFicheCount());
        primitivesWriter.writeInt(ficheSearchResult.getFicheMaximum());
        List<MotcleSearchResultInfo> motcleInfoList = ficheSearchResult.getMotcleSearchResultInfoList();
        int motcleLength = motcleInfoList.size();
        primitivesWriter.writeInt(motcleLength);
        Codes corpusCodes = ficheSearchResult.getCorpusCodes();
        int corpusCount = corpusCodes.getCodeCount();
        primitivesWriter.writeInt(corpusCount);
        for (int i = 0; i < corpusCount; i++) {
            primitivesWriter.writeInt(corpusCodes.getCode(i));
        }
        List<FicheSearchResultGroup> groupList = ficheSearchResult.getFicheSearchResultGroupList();
        int groupLength = groupList.size();
        primitivesWriter.writeInt(groupLength);
        for (int i = 0; i < groupLength; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = groupList.get(i);
            primitivesWriter.writeInt(ficheSearchResultGroup.getCategoryRank());
            List<FicheSearchResultInfo> ficheInfoList = ficheSearchResultGroup.getFicheSearchResultInfoList();
            int ficheLength = ficheInfoList.size();
            primitivesWriter.writeInt(ficheLength);
            for (int j = 0; j < ficheLength; j++) {
                writeFicheSearchResultInfo(primitivesWriter, ficheInfoList.get(j));
            }
        }
        for (int i = 0; i < motcleLength; i++) {
            writeMotcleSearchResultInfo(primitivesWriter, motcleInfoList.get(i));
        }
    }

    public static FicheSearchResult read(File file) throws IOException {
        DataInputStream dataInput = new DataInputStream(new FileInputStream(file));
        DataInputPrimitivesReader primReader = new DataInputPrimitivesReader(dataInput);
        return FicheSearchResultStorage.read(primReader);
    }

    public static FicheSearchResult read(PrimitivesReader primitivesReader) throws IOException {
        int version = primitivesReader.readInt();
        if (version != CACHE_VERSION) {
            return null;
        }
        FicheSearchSource ficheSearchSource = readFicheSearchSource(primitivesReader);
        FicheSearchResultBuilder builder = new FicheSearchResultBuilder(ficheSearchSource, false);
        int ficheCount = primitivesReader.readInt();
        builder.setFicheCount(ficheCount);
        builder.setFicheMaximum(primitivesReader.readInt());
        int motcleCount = primitivesReader.readInt();
        int corpusCount = primitivesReader.readInt();
        for (int i = 0; i < corpusCount; i++) {
            builder.addCodecorpus(primitivesReader.readInt());
        }
        int ficheSearchResultGroupCount = primitivesReader.readInt();
        if (ficheCount > 0) {
            for (int i = 0; i < ficheSearchResultGroupCount; i++) {
                FicheSearchResultGroupBuilder groupBuilder = new FicheSearchResultGroupBuilder();
                groupBuilder.setCategoryRank(primitivesReader.readInt());
                int count = primitivesReader.readInt();
                groupBuilder.ensureInfoCapacity(count);
                for (int j = 0; j < count; j++) {
                    groupBuilder.addFicheSearchResultInfo(readFicheSearchResultInfo(primitivesReader));
                }
                builder.addFicheSearchResultGroup(groupBuilder.toFicheSearchResultGroup());
            }
        }
        if (motcleCount > 0) {
            for (int i = 0; i < motcleCount; i++) {
                builder.addMotcleSearchResultInfo(readMotcleSearchResultInfo(primitivesReader));
            }
        }
        return builder.toFicheSearchResult();
    }

    private static void writeFicheSearchResultInfo(PrimitivesWriter primitivesWriter, FicheSearchResultInfo ficheSearchResultInfo) throws IOException {
        primitivesWriter.writeInt(ficheSearchResultInfo.getCode());
        Codes motcleCodes = ficheSearchResultInfo.getMotcleCodes();
        int motcleCount = motcleCodes.getCodeCount();
        primitivesWriter.writeInt(motcleCount);
        for (int i = 0; i < motcleCount; i++) {
            primitivesWriter.writeInt(motcleCodes.getCode(i));
        }
        List<InAlineaOccurrences> occurrencesList = ficheSearchResultInfo.getInAlineaOccurrencesList();
        int occurrencesLength = occurrencesList.size();
        primitivesWriter.writeInt(occurrencesLength);
        for (int i = 0; i < occurrencesLength; i++) {
            InAlineaOccurrences inAlineaOccurrenceList = occurrencesList.get(i);
            primitivesWriter.writeInt(inAlineaOccurrenceList.getAlineaInteger());
            writeSearchTokenPositionList(primitivesWriter, inAlineaOccurrenceList.getSearchTokenOccurrenceList());
        }
        int[] scoreArray = ficheSearchResultInfo.getScoreArray();
        int scoreLength = scoreArray.length;
        primitivesWriter.writeInt(scoreLength);
        for (int i = 0; i < scoreLength; i++) {
            primitivesWriter.writeInt(scoreArray[i]);
        }
    }

    private static void writeMotcleSearchResultInfo(PrimitivesWriter primitivesWriter, MotcleSearchResultInfo motcleSearchResultInfo) throws IOException {
        primitivesWriter.writeInt(motcleSearchResultInfo.getCode());
        List<InLangOccurrences> occurrencesList = motcleSearchResultInfo.getInLangOccurrencesList();
        int occurrencesLength = occurrencesList.size();
        primitivesWriter.writeInt(occurrencesLength);
        for (int i = 0; i < occurrencesLength; i++) {
            InLangOccurrences inLangOccurrences = occurrencesList.get(i);
            primitivesWriter.writeString(inLangOccurrences.getLang().toString());
            writeSearchTokenPositionList(primitivesWriter, inLangOccurrences.getSearchTokenOccurrenceList());
        }
    }

    private static void writeSearchTokenPositionList(PrimitivesWriter primitivesWriter, List<SearchTokenOccurrence> searchTokenOccurrenceList) throws IOException {
        int size = searchTokenOccurrenceList.size();
        primitivesWriter.writeInt(size);
        for (int j = 0; j < size; j++) {
            SearchTokenOccurrence tokenOccurrence = searchTokenOccurrenceList.get(j);
            primitivesWriter.writeInt(tokenOccurrence.getOperandNumber());
            primitivesWriter.writeInt(tokenOccurrence.getBeginIndex());
            primitivesWriter.writeInt(tokenOccurrence.getLength());
        }
    }

    private static FicheSearchResultInfo readFicheSearchResultInfo(PrimitivesReader primitivesReader) throws IOException {
        int code = primitivesReader.readInt();
        int motcleCount = primitivesReader.readInt();
        int[] motcleCodeArray = null;
        if (motcleCount > 0) {
            motcleCodeArray = new int[motcleCount];
            for (int i = 0; i < motcleCount; i++) {
                motcleCodeArray[i] = primitivesReader.readInt();
            }
        }
        int inAlineaOccurrenceListCount = primitivesReader.readInt();
        InAlineaOccurrences[] inAlineaOccurrenceListArray = null;
        if (inAlineaOccurrenceListCount > 0) {
            inAlineaOccurrenceListArray = new InAlineaOccurrences[inAlineaOccurrenceListCount];
            for (int i = 0; i < inAlineaOccurrenceListCount; i++) {
                int alineaInteger = primitivesReader.readInt();
                SearchTokenOccurrence[] searchTokenOccurrenceArray = readSearchTokenPositionArray(primitivesReader);
                inAlineaOccurrenceListArray[i] = ResultUtils.toInAlineaOccurrences(alineaInteger, searchTokenOccurrenceArray);
            }
        }
        int scoreLength = primitivesReader.readInt();
        int[] scoreArray = new int[scoreLength];
        for (int i = 0; i < scoreLength; i++) {
            scoreArray[i] = primitivesReader.readInt();
        }
        return FicheSearchResultInfoBuilder.toFicheSearchResultInfo(code, motcleCodeArray, inAlineaOccurrenceListArray, scoreArray);
    }

    private static MotcleSearchResultInfo readMotcleSearchResultInfo(PrimitivesReader primitivesReader) throws IOException {
        int code = primitivesReader.readInt();
        int occurrencesLength = primitivesReader.readInt();
        InLangOccurrences[] inLangOccurrencesArray = null;
        if (occurrencesLength > 0) {
            inLangOccurrencesArray = new InLangOccurrences[occurrencesLength];
            for (int i = 0; i < occurrencesLength; i++) {
                Lang lang = Lang.build(primitivesReader.readString());
                SearchTokenOccurrence[] searchTokenOccurrenceArray = readSearchTokenPositionArray(primitivesReader);
                inLangOccurrencesArray[i] = ResultUtils.toInLangOccurrences(lang, searchTokenOccurrenceArray);
            }
        }
        return MotcleSearchResultBuilder.toMotcleSearchResultInfo(code, inLangOccurrencesArray);
    }

    private static SearchTokenOccurrence[] readSearchTokenPositionArray(PrimitivesReader primitivesReader) throws IOException {
        int searchTokenPositionCount = primitivesReader.readInt();
        if (searchTokenPositionCount == 0) {
            return null;
        }
        SearchTokenOccurrence[] result = new SearchTokenOccurrence[searchTokenPositionCount];
        for (int i = 0; i < searchTokenPositionCount; i++) {
            int operandNumber = primitivesReader.readInt();
            int beginIndex = primitivesReader.readInt();
            int length = primitivesReader.readInt();
            result[i] = SearchTokenOccurrenceBuilder.toSearchTokenOccurrence(operandNumber, beginIndex, length);
        }
        return result;
    }

    private static void writeFicheSearchSource(PrimitivesWriter primitivesWriter, FicheSearchSource ficheSearchSource) throws IOException {
        QId qId = ficheSearchSource.getQId();
        String qIdString;
        if (qId != null) {
            qIdString = qId.toString();
        } else {
            qIdString = "";
        }
        primitivesWriter.writeString(qIdString);
        primitivesWriter.writeString(ficheSearchSource.getCanonicalQ().toString());
        SearchOptionsDef searchOptionsDef = ficheSearchSource.getSearchOptionsDef();
        if (searchOptionsDef == null) {
            primitivesWriter.writeBoolean(false);
            return;
        } else {
            primitivesWriter.writeBoolean(true);
        }
        Lang lang = searchOptionsDef.getLang();
        if (lang != null) {
            primitivesWriter.writeString(lang.toString());
        } else {
            primitivesWriter.writeString("");
        }
        List<Lang> filterLangList = searchOptionsDef.getFilterLangList();
        primitivesWriter.writeInt(filterLangList.size());
        for (Lang filterLang : filterLangList) {
            primitivesWriter.writeString(filterLang.toString());
        }
        writeListReduction(primitivesWriter, searchOptionsDef, ScrutariDataURI.BASEURI_TYPE);
        writeListReduction(primitivesWriter, searchOptionsDef, ScrutariDataURI.CORPUSURI_TYPE);
        writeListReduction(primitivesWriter, searchOptionsDef, ScrutariDataURI.THESAURUSURI_TYPE);
        writeListReduction(primitivesWriter, searchOptionsDef, ScrutariDataURI.FICHEURI_TYPE);
        writeListReduction(primitivesWriter, searchOptionsDef, ScrutariDataURI.MOTCLEURI_TYPE);
        ScrutariAttributes.toPrimitives(searchOptionsDef, primitivesWriter);
    }

    private static void writeListReduction(PrimitivesWriter primitivesWriter, SearchOptionsDef searchOptionsDef, short type) throws IOException {
        ListReduction listReduction = searchOptionsDef.getListReduction(type);
        if (listReduction == null) {
            primitivesWriter.writeInt(0);
        } else {
            List<ScrutariDataURI> uriList = listReduction.getScrutariDataURIList();
            primitivesWriter.writeInt(uriList.size());
            if (!uriList.isEmpty()) {
                primitivesWriter.writeBoolean(listReduction.isExclude());
                for (ScrutariDataURI uri : uriList) {
                    URIPrimitivesUtils.writeScrutariDataURI(uri, primitivesWriter);
                }
            }
        }
    }

    private static FicheSearchSource readFicheSearchSource(PrimitivesReader primitivesReader) throws IOException {
        String qIdString = primitivesReader.readString();
        QId qId = null;
        if (qIdString.length() > 0) {
            try {
                qId = QId.parse(qIdString);
            } catch (ParseException pe) {
            }
        }
        String q = primitivesReader.readString();
        CanonicalQ canonicalQ = CanonicalQ.newInstance(q);
        boolean withOptions = primitivesReader.readBoolean();
        if (!withOptions) {
            return FicheSearchSourceBuilder.build(qId, canonicalQ, null);
        }
        SearchOptionsDefBuilder searchOptionsDefBuilder = new SearchOptionsDefBuilder();
        String langString = primitivesReader.readString();
        if (!langString.isEmpty()) {
            searchOptionsDefBuilder.setLang(Lang.build(langString));
        }
        int langCount = primitivesReader.readInt();
        for (int i = 0; i < langCount; i++) {
            searchOptionsDefBuilder.addFilterLang(Lang.build(primitivesReader.readString()));
        }
        readListReduction(primitivesReader, searchOptionsDefBuilder, ScrutariDataURI.BASEURI_TYPE);
        readListReduction(primitivesReader, searchOptionsDefBuilder, ScrutariDataURI.CORPUSURI_TYPE);
        readListReduction(primitivesReader, searchOptionsDefBuilder, ScrutariDataURI.THESAURUSURI_TYPE);
        readListReduction(primitivesReader, searchOptionsDefBuilder, ScrutariDataURI.FICHEURI_TYPE);
        readListReduction(primitivesReader, searchOptionsDefBuilder, ScrutariDataURI.MOTCLEURI_TYPE);
        ScrutariAttributes.fromPrimitives(primitivesReader, searchOptionsDefBuilder.getAttributesBuilder());
        return FicheSearchSourceBuilder.build(qId, canonicalQ, searchOptionsDefBuilder.toSearchOptionsDef());
    }

    private static void readListReduction(PrimitivesReader primitivesReader, SearchOptionsDefBuilder searchOptionsDefBuilder, short type) throws IOException {
        int size = primitivesReader.readInt();
        if (size == 0) {
            return;
        }
        ListReductionBuilder listReductionBuilder = new ListReductionBuilder(type);
        boolean exclude = primitivesReader.readBoolean();
        listReductionBuilder.setExclude(exclude);
        for (int i = 0; i < size; i++) {
            listReductionBuilder.add(URIPrimitivesUtils.readScrutariDataURI(primitivesReader));
        }
        searchOptionsDefBuilder.putListReduction(listReductionBuilder.toListReduction());
    }

}
