/* ScrutariLib_Xml - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.xml.xmlpull;

import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.text.ParseException;
import java.util.Map;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.xml.xmlpull.DefaultElementHandler;
import net.mapeadores.util.xml.xmlpull.LabelElementHandler;
import net.mapeadores.util.xml.xmlpull.SimpleElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import net.scrutari.data.CorpusData;
import net.scrutari.data.DataConstants;
import net.scrutari.data.DataValidationException;
import net.scrutari.data.DataValidator;
import net.scrutari.data.FicheData;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.db.api.DataCoder;
import net.scrutari.db.api.caches.DataCacheWriter;
import net.scrutari.xml.Parameters;


/**
 *
 * @author Vincent Calame
 */
public class CorpusHandler extends DefaultElementHandler {

    private static final int HREF_PARENT = 1;
    private static final int TITRE = 3;
    private static final int SOUSTITRE = 4;
    private static final int DATE = 5;
    private static final int LANG = 6;
    private static final int HREF = 7;
    private static final int COMPLEMENT = 8;
    private static final int FICHE_ICON = 9;
    private static final int LATITUDE = 10;
    private static final int LONGITUDE = 11;
    private final DataCacheWriter dataCacheWriter;
    private final DataCoder dataCoder;
    private final DataValidator dataValidator;
    private final MessageHandler messageHandler;
    private final CorpusURI corpusURI;
    private final int codebase;
    private final int codecorpus;
    private final CorpusData.Builder corpusDataBuilder;
    private final FicheHandler ficheHandler = new FicheHandler();
    private final String corpusxpath;
    private boolean metadataDone = false;

    public CorpusHandler(CorpusURI corpusURI, Parameters parameters) {
        this.corpusURI = corpusURI;
        String corpusName = corpusURI.getCorpusName();
        this.messageHandler = parameters.getMessageHandler();
        this.dataValidator = parameters.getDataValidator();
        this.corpusxpath = "/base/corpus[@corpus-name='" + corpusName + "']";
        this.dataCacheWriter = parameters.getDataCacheWriter();
        this.dataCoder = parameters.getDataCoder();
        if (dataCoder.checkCode(corpusURI) != -1) {
            CorpusHandler.this.warning("_ error.existing.xml.attributevalue", corpusxpath, "corpus-name", corpusName);
            throw new IllegalArgumentException();
        }
        this.codecorpus = dataCoder.getCode(corpusURI);
        this.codebase = parameters.getCodeBase();
        corpusDataBuilder = new CorpusData.Builder(corpusURI);
    }

    @Override
    public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
        if (!metadataDone) {
            if (tagname.equals("corpus-metadata")) {
                metadataDone = true;
                return new MetadataHandler();
            } else {
                CorpusHandler.this.warning("_ error.unsupported.xml.missingtag", corpusxpath, "corpus-metadata");
                return null;
            }
        } else {
            if (tagname.equals("fiche")) {
                String ficheId = getAttributeString(attributeMap, "fiche-id");
                if (ficheId == null) {
                    CorpusHandler.this.warning("_ error.empty.xml.attribute", corpusxpath + "/fiche", "fiche-id");
                    return null;
                }
                String ficheXpath = corpusxpath + "/fiche[@fiche-id='" + ficheId + "']";
                FicheURI ficheURI = FicheURI.check(corpusURI, ficheId);
                if (ficheURI == null) {
                    CorpusHandler.this.warning("_ error.wrong.xml.attributevalue", corpusxpath + "/fiche", "fiche-id", ficheId);
                    return null;
                }
                if (dataCoder.checkCode(ficheURI) != -1) {
                    CorpusHandler.this.warning("_ error.existing.xml.attributevalue", ficheXpath, "fiche-id", ficheId);
                    return null;
                }
                ficheHandler.newFiche(ficheURI, ficheXpath);
                return ficheHandler;
            } else {
                return null;
            }
        }
    }

    @Override
    public void endSubElement(XmlPullElementHandler elementHandler) {
        if (elementHandler instanceof FicheHandler) {
            FicheHandler fh = (FicheHandler) elementHandler;
            FicheData.Builder ficheDataBuilder = fh.getFicheDataBuilder();
            try {
                ficheDataBuilder.validate(dataValidator);
                int codefiche = dataCoder.getCode(fh.getFicheURI());
                FicheData ficheData = ficheDataBuilder.toFicheData(codefiche, corpusDataBuilder.getHrefParent(), codecorpus, codebase);
                dataCacheWriter.cacheFicheData(ficheData);
                corpusDataBuilder.addCodeFiche(codefiche);
            } catch (DataValidationException dve) {

            }
        }
    }

    public CorpusData flushCorpusData() {
        if (!metadataDone) {
            return null;
        }
        CorpusData corpusData = corpusDataBuilder.toCorpusData(codecorpus, codebase);
        dataCacheWriter.cacheCorpusData(corpusData);
        return corpusData;
    }

    private void warning(String key, Object... values) {
        XmlPullUtils.warning(messageHandler, key, values);
    }

    private void warning(Message message) {
        XmlPullUtils.warning(messageHandler, message);
    }


    private class MetadataHandler extends DefaultElementHandler {

        private MetadataHandler() {

        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            String xpath = corpusxpath + "/corpus-metadata/" + tagname;
            switch (tagname) {
                case "intitule-corpus":
                    return new PhraseHandler(DataConstants.CORPUS_TITLE, xpath);
                case "intitule-fiche":
                    return new PhraseHandler(DataConstants.CORPUS_FICHE, xpath);
                case "complement-metadata":
                    int complementNumber = corpusDataBuilder.createNewComplement();
                    return new PhraseHandler(DataConstants.CORPUS_COMPLEMENTPREFIX + complementNumber, xpath);
                case "href-parent":
                    return new SimpleElementHandler(HREF_PARENT);
                case "corpus-icon":
                    return null;
                case "attr":
                    AttributeKey attributeKey = XmlPullUtils.parseAttributeKey(attributeMap, messageHandler, xpath);
                    if (attributeKey != null) {
                        return new AttrHandler(attributeKey);
                    } else {
                        return null;
                    }
                default:
                    return null;
            }
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            switch (type) {
                case HREF_PARENT:
                    boolean hrefParentDone = false;
                    try {
                        hrefParentDone = corpusDataBuilder.setHrefParent(XmlPullUtils.testURI(text));
                    } catch (MalformedURLException | URISyntaxException e) {

                    }
                    if (!hrefParentDone) {
                        CorpusHandler.this.warning("_ error.wrong.xml.tagvalue", corpusxpath + "/corpus-metadata/href-parent", text);
                    }
                    break;
            }
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof AttrHandler) {
                AttrHandler attrHandler = (AttrHandler) elementHandler;
                corpusDataBuilder.addAttribute(attrHandler.getAttributeKey(), attrHandler.getValueList());
            }
        }

    }


    private class FicheHandler extends DefaultElementHandler {

        private String ficheXpath;
        private FicheURI ficheURI;
        private FicheData.Builder ficheDataBuilder;

        private FicheHandler() {
        }

        private void newFiche(FicheURI ficheURI, String ficheXpath) {
            this.ficheURI = ficheURI;
            this.ficheXpath = ficheXpath;
            ficheDataBuilder = new FicheData.Builder(ficheURI, corpusDataBuilder.getComplementMaxNumber());
        }

        private FicheData.Builder getFicheDataBuilder() {
            return ficheDataBuilder;
        }

        private FicheURI getFicheURI() {
            return ficheURI;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            switch (tagname) {
                case "titre":
                    return new SimpleElementHandler(TITRE);
                case "soustitre":
                    return new SimpleElementHandler(SOUSTITRE);
                case "date":
                    return new SimpleElementHandler(DATE);
                case "lang":
                    return new SimpleElementHandler(LANG);
                case "href":
                    return new SimpleElementHandler(HREF);
                case "complement":
                    return new SimpleElementHandler(COMPLEMENT);
                case "fiche-icon":
                    return new SimpleElementHandler(FICHE_ICON);
                case "geoloc":
                    return new GeolocHandler();
                case "attr":
                    AttributeKey attributeKey = XmlPullUtils.parseAttributeKey(attributeMap, messageHandler, ficheXpath + "/attr");
                    if (attributeKey != null) {
                        return new AttrHandler(attributeKey);
                    } else {
                        return null;
                    }
                default:
                    return null;
            }
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            switch (type) {
                case TITRE:
                    ficheDataBuilder.setTitre(text);
                    break;
                case SOUSTITRE:
                    ficheDataBuilder.setSoustitre(text);
                    break;
                case DATE:
                    try {
                        if ((text != null) && (text.length() != 0)) {
                            ficheDataBuilder.setDate(FuzzyDate.parse(text));
                        }
                    } catch (ParseException pe) {
                        warning("_ error.wrong.xml.tagvalue", ficheXpath + "/date", text);
                    }
                    break;
                case LANG:
                    try {
                        if ((text != null) && (text.length() != 0)) {
                            ficheDataBuilder.setLang(Lang.parse(text));
                        }
                    } catch (ParseException pe) {
                        warning("_ error.wrong.xml.tagvalue", ficheXpath + "/lang", text);
                    }
                    break;
                case HREF:
                    boolean hrefDone = false;
                    try {
                        hrefDone = ficheDataBuilder.setHref(XmlPullUtils.testURI(text));
                    } catch (MalformedURLException | URISyntaxException e) {

                    }
                    if (!hrefDone) {
                        warning("_ error.wrong.xml.tagvalue", ficheXpath + "/href", text);
                    }
                    break;
                case COMPLEMENT:
                    boolean done = ficheDataBuilder.addComplement(text);
                    if (!done) {
                        warning("_ error.unsupported.xml.toomanytags", ficheXpath + "/complement");
                    }
                    break;
                case FICHE_ICON:
                    boolean ficheIconDone = false;
                    try {
                        ficheIconDone = ficheDataBuilder.setFicheIcon(XmlPullUtils.testURI(text));
                    } catch (MalformedURLException | URISyntaxException e) {

                    }
                    if (!ficheIconDone) {
                        warning("_ error.wrong.xml.tagvalue", ficheXpath + "/fiche-icon", text);
                    }
                    break;
            }
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof GeolocHandler) {
                GeolocHandler geolocHandler = (GeolocHandler) elementHandler;
                String latitude = geolocHandler.latitude;
                String longitude = geolocHandler.longitude;
                boolean done = true;
                if (latitude == null) {
                    done = false;
                    warning("_ error.unsupported.xml.missingtag", ficheXpath + "/geoloc", "lat");
                }
                if (longitude == null) {
                    done = false;
                    warning("_ error.unsupported.xml.missingtag", ficheXpath + "/geoloc", "lon");
                }
                if (done) {
                    done = ficheDataBuilder.setGeoloc(latitude, longitude);
                    if (!done) {
                        warning("_ error.wrong.xml.tagvalue", ficheXpath + "/geoloc/[lat OR lon]", latitude + " / " + longitude);
                    }
                }
            } else if (elementHandler instanceof AttrHandler) {
                AttrHandler attrHandler = (AttrHandler) elementHandler;
                ficheDataBuilder.addAttribute(attrHandler.getAttributeKey(), attrHandler.getValueList());
            }
        }

    }


    private class PhraseHandler extends DefaultElementHandler {

        private final String name;
        private final String xpath;

        private PhraseHandler(String name, String xpath) {
            this.name = name;
            this.xpath = xpath;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LabelElementHandler(attributeMap, xpath + "/lib");
                } catch (ErrorMessageException eme) {
                    warning(eme.getErrorMessage());
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            corpusDataBuilder.addLabel(name, XmlPullUtils.testLabel(elementHandler));
        }

    }


    private class GeolocHandler extends DefaultElementHandler {

        private String latitude;
        private String longitude;

        private GeolocHandler() {
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            switch (tagname) {
                case "lat":
                    return new SimpleElementHandler(LATITUDE);
                case "lon":
                    return new SimpleElementHandler(LONGITUDE);
                default:
                    return null;
            }
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            switch (type) {
                case LATITUDE:
                    latitude = text;
                    break;
                case LONGITUDE:
                    longitude = text;
                    break;
            }
        }

    }


}
