/* SctServer - Copyright (c) 2014-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.impl;

import fr.exemole.sctserver.api.ScrutariSourceManager;
import fr.exemole.sctserver.api.collect.CollectResult;
import fr.exemole.sctserver.api.collect.ScrutariSource;
import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import fr.exemole.sctserver.api.conf.ConfDirectory;
import fr.exemole.sctserver.api.storage.EngineStorage;
import fr.exemole.sctserver.tools.collect.CollectEngine;
import fr.exemole.sctserver.tools.collect.CollectUtils;
import fr.exemole.sctserver.tools.collect.ScrutariSourceStorage;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.logging.MultiMessageHandler;
import net.mapeadores.util.primitives.FuzzyDate;


/**
 *
 * @author Vincent Calame
 */
class ScrutariSourceManagerImpl implements ScrutariSourceManager {

    private final Map<String, InternalScrutariSource> scrutariSourceMap = new LinkedHashMap<String, InternalScrutariSource>();
    private List<ScrutariSource> scrutariSourceList = CollectUtils.EMPTY_SCRUTARISOURCELIST;
    private EngineStorage engineStorage;

    ScrutariSourceManagerImpl(EngineStorage engineStorage) {
        this.engineStorage = engineStorage;
    }

    @Override
    public ScrutariSource getScrutariSouce(String scrutariSourceName) {
        return scrutariSourceMap.get(scrutariSourceName);
    }

    @Override
    public List<ScrutariSource> getScrutariSourceList() {
        return scrutariSourceList;
    }

    void reloadConf() {
        ConfDirectory confDirectory = engineStorage.getConfDirectory();
        ScrutariSourceDef[] scrutariSourceDefArray = confDirectory.getScrutariSourceDefArray();
        if (scrutariSourceDefArray == null) {
            scrutariSourceMap.clear();
        } else if (scrutariSourceMap.isEmpty()) {
            Map<String, ScrutariSourceStorage.Status> statusMap = ScrutariSourceStorage.fromString(engineStorage.getContent(EngineStorage.STATUS_NAME));
            int length = scrutariSourceDefArray.length;
            for (int i = 0; i < length; i++) {
                ScrutariSourceDef scrutariSourceDef = scrutariSourceDefArray[i];
                String name = scrutariSourceDef.getName();
                InternalScrutariSource scrutariSource = new InternalScrutariSource(scrutariSourceDef);
                ScrutariSourceStorage.Status status = statusMap.get(name);
                if (status != null) {
                    scrutariSource.update(status);
                }
                scrutariSourceMap.put(name, scrutariSource);
            }
        } else {
            Map<String, InternalScrutariSource> copyMap = new HashMap<String, InternalScrutariSource>();
            copyMap.putAll(scrutariSourceMap);
            scrutariSourceMap.clear();
            int length = scrutariSourceDefArray.length;
            for (int i = 0; i < length; i++) {
                ScrutariSourceDef scrutariSourceDef = scrutariSourceDefArray[i];
                String name = scrutariSourceDef.getName();
                InternalScrutariSource scrutariSource = copyMap.get(name);
                if (scrutariSource != null) {
                    scrutariSource.setScrutariSourceDef(scrutariSourceDef);
                } else {
                    scrutariSource = new InternalScrutariSource(scrutariSourceDef);
                }
                scrutariSourceMap.put(name, scrutariSource);
            }
        }
        if (scrutariSourceMap.isEmpty()) {
            scrutariSourceList = CollectUtils.EMPTY_SCRUTARISOURCELIST;
        } else {
            scrutariSourceList = CollectUtils.wrap(scrutariSourceMap.values().toArray(new ScrutariSource[scrutariSourceMap.size()]));
        }
        store();
    }

    @Override
    public boolean collect(MultiMessageHandler messageHandler, boolean forceDataDownload) {
        if (scrutariSourceMap.isEmpty()) {
            return false;
        }
        boolean dataChange = false;
        for (InternalScrutariSource scrutariSource : scrutariSourceMap.values()) {
            CollectResult collectResult = CollectEngine.collectData(scrutariSource, engineStorage, messageHandler, forceDataDownload);
            short currentState = scrutariSource.getState();
            short newState = collectResult.getState();
            boolean update = false;
            if (currentState != newState) {
                update = true;
            }
            if (newState == CollectResult.OK_STATE) {
                if ((currentState != CollectResult.OK_STATE) || (FuzzyDate.compare(collectResult.getLastUpdate(), scrutariSource.getLastUpdate()) != 0)) {
                    dataChange = true;
                    update = true;
                }
            }
            if (update) {
                scrutariSource.update(collectResult);
            }
        }
        store();
        if (forceDataDownload) {
            return true;
        } else {
            return dataChange;
        }
    }

    boolean setState(ScrutariSource scrutariSource, short state) {
        boolean done = ((InternalScrutariSource) scrutariSource).setState(state);
        if (done) {
            store();
        }
        return done;
    }

    private void store() {
        engineStorage.saveContent(EngineStorage.STATUS_NAME, ScrutariSourceStorage.toString(scrutariSourceList));
    }


    private class InternalScrutariSource implements ScrutariSource {

        private ScrutariSourceDef scrutariSourceDef;
        private short state = CollectResult.UNKNOWN_STATE;
        private FuzzyDate lastUpdate = null;

        private InternalScrutariSource(ScrutariSourceDef scrutariSourceDef) {
            this.scrutariSourceDef = scrutariSourceDef;
        }

        private InternalScrutariSource(ScrutariSource scrutariSource) {
            this.scrutariSourceDef = scrutariSource.getScrutariSourceDef();
            this.lastUpdate = scrutariSource.getLastUpdate();
            this.state = scrutariSource.getState();
        }

        @Override
        public ScrutariSourceDef getScrutariSourceDef() {
            return scrutariSourceDef;
        }

        @Override
        public short getState() {
            return state;
        }

        @Override
        public FuzzyDate getLastUpdate() {
            return lastUpdate;
        }

        private void setScrutariSourceDef(ScrutariSourceDef scrutariSourceDef) {
            if (!areUrlEqual(this.scrutariSourceDef, scrutariSourceDef)) {
                state = CollectResult.UNKNOWN_STATE;
                lastUpdate = null;
            }
            this.scrutariSourceDef = scrutariSourceDef;
        }

        private boolean setState(short state) {
            if (this.state == state) {
                return false;
            }
            this.state = state;
            return true;
        }

        private void update(CollectResult collectResult) {
            this.state = collectResult.getState();
            this.lastUpdate = collectResult.getLastUpdate();
        }

    }

    private static boolean areUrlEqual(ScrutariSourceDef ssd1, ScrutariSourceDef ssd2) {
        List<ScrutariSourceDef.UrlEntry> entryList1 = ssd1.getUrlEntryList();
        List<ScrutariSourceDef.UrlEntry> entryList2 = ssd2.getUrlEntryList();
        int size1 = entryList1.size();
        int size2 = entryList2.size();
        if (size1 != size2) {
            return false;
        }
        for (int i = 0; i < size1; i++) {
            if (!entryList1.get(i).getUrl().equals(entryList2.get(i).getUrl())) {
                return false;
            }
        }
        return true;
    }

}
