/* SctEngine - Copyright (c) 2014-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.impl;

import fr.exemole.sctserver.api.AttributeDefManager;
import fr.exemole.sctserver.api.ConfState;
import fr.exemole.sctserver.api.EngineContext;
import fr.exemole.sctserver.api.EngineMetadata;
import fr.exemole.sctserver.api.FieldVariantManager;
import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.ScrutariSourceManager;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.api.collect.CollectResult;
import fr.exemole.sctserver.api.collect.ScrutariSource;
import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import fr.exemole.sctserver.api.conf.ConfConstants;
import fr.exemole.sctserver.api.conf.ConfDirectory;
import fr.exemole.sctserver.api.conf.ConfFileInfo;
import fr.exemole.sctserver.api.conf.ConfIOException;
import fr.exemole.sctserver.api.log.SctLogConstants;
import fr.exemole.sctserver.api.storage.EngineStorage;
import fr.exemole.sctserver.tools.DataValidatorBuilder;
import fr.exemole.sctserver.tools.log.UpdateLogHandler;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.logging.CategorizedMessageEntry;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.logging.MessageLog;
import net.mapeadores.util.logging.TimeLog;
import net.scrutari.data.BaseData;
import net.scrutari.data.DataValidator;
import net.scrutari.datauri.BaseURI;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataCoder;
import net.scrutari.db.api.HistoryDataCoder;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.ScrutariDBName;
import net.scrutari.db.api.caches.DataCacheReaderFactory;
import net.scrutari.db.api.caches.DataCacheWriter;
import net.scrutari.db.tools.AliasManager;
import net.scrutari.db.tools.DataCoderUtils;
import net.scrutari.db.tools.HistoryDataCoderBuilder;
import net.scrutari.db.tools.ScrutariDBBuilder;
import net.scrutari.xml.BaseURIXmlEngine;
import net.scrutari.xml.BaseXmlEngine;
import net.scrutari.xml.IndexationXmlEngine;
import net.scrutari.xml.Parameters;
import net.scrutari.xml.ScrutariXmlConstants;
import net.scrutari.xml.xmlpull.XmlChecker;
import org.xmlpull.v1.XmlPullParserException;


/**
 *
 * @author Vincent Calame
 */
public final class SctEngineImpl implements SctEngine {

    private final String name;
    private final EngineContext engineContext;
    private final EngineStorage engineStorage;
    private final ConfDirectory confDirectory;
    private final ScrutariSourceManagerImpl scrutariSourceManager;
    private final EngineMetadataImpl engineMetadata;
    private final FieldVariantManagerImpl fieldVariantManager;
    private final AttributeDefManagerImpl attributeDefManager;
    private ScrutariSessionImpl scrutariSession;
    private final ConfStateImpl confState = new ConfStateImpl();
    private boolean onReload = false;
    private String lastName = null;

    public SctEngineImpl(String name, EngineStorage engineStorage, EngineContext engineContext) {
        this.name = name;
        this.engineContext = engineContext;
        this.engineStorage = engineStorage;
        this.confDirectory = engineStorage.getConfDirectory();
        this.engineMetadata = new EngineMetadataImpl();
        this.scrutariSourceManager = new ScrutariSourceManagerImpl(engineStorage);
        this.fieldVariantManager = new FieldVariantManagerImpl();
        this.attributeDefManager = new AttributeDefManagerImpl();
        init();
    }

    private void init() {
        EngineStorage.CacheCheck cacheCheck = engineStorage.checkLastCache(null);
        boolean done = false;
        if (cacheCheck != null) {
            AliasManager aliasManager = (AliasManager) engineStorage.getCachedObject(cacheCheck.getScrutariDBName(), AliasManager.class);
            if (aliasManager != null) {
                TimeLog timeLog = new TimeLog();
                reloadConfDirectory();
                timeLog.log("confdirectory_init");
                scrutariSourceManager.reloadConf();
                timeLog.log("scrutarisourcemanager_loaded");
                AttributeKeyCoder attributeKeyCoder = AttributeDefManagerImpl.toAttributeKeyCoder(confDirectory.getAttributeDefArrayMap());
                timeLog.log("attributekeycoder_build");
                ScrutariDB newScrutariDB = ScrutariDBBuilder.build(cacheCheck.getScrutariDBName(), attributeKeyCoder, cacheCheck.getDataCacheReaderFactory(), cacheCheck.getLexieCacheReaderFactory(), aliasManager, timeLog);
                timeLog.log("scrutaridb_build");
                scrutariSession = ScrutariSessionImpl.newInstance(this, newScrutariDB, aliasManager, confDirectory, timeLog);
                timeLog.log("scrutarisession_build");
                timeLog.log("start_end");
                engineStorage.getLogStorage().log(SctLogConstants.START_LOG, "time.txt", timeLog.toString());
                done = true;
            }
        }
        if (!done) {
            reload(INIT_TYPE);
        }
    }

    @Override
    public boolean reload(short type) {
        if (onReload) {
            return false;
        }
        if (lastName != null) {
            String currentName = ScrutariDBName.getCurrentName();
            if (currentName.equals(lastName)) {
                return false;
            }
        }
        onReload = true;
        reloadConfDirectory();
        scrutariSourceManager.reloadConf();
        if (type != UPDATECONF_TYPE) {
            UpdateLogHandler updateLogHandler = new UpdateLogHandler();
            TimeLog timeLog = new TimeLog();
            boolean forceReload = (type == FORCE_TYPE);
            boolean done = scrutariSourceManager.collect(updateLogHandler, forceReload);
            timeLog.log("collect_done");
            if ((type != CRON_TYPE) || (done)) {
                ScrutariDBName newScrutariDBName = reinitScrutariDB(updateLogHandler, timeLog);
                lastName = newScrutariDBName.toString();
                timeLog.log("update_end");
                engineStorage.getLogStorage().log(SctLogConstants.UPDATE_LOG, "time.txt", timeLog.toString());
                MessageLog messageLog = updateLogHandler.toMessageLog();
                String dataLogContent = null;
                if (!messageLog.getLogGroupList().isEmpty()) {
                    dataLogContent = LogUtils.toXmlString(messageLog);
                }
                engineStorage.getLogStorage().log(SctLogConstants.UPDATE_LOG, "data.xml", dataLogContent);
            }
        } else {
            scrutariSession.reloadConf(confDirectory);
        }
        onReload = false;
        return true;
    }

    @Override
    public String getEngineName() {
        return name;
    }

    @Override
    public EngineContext getEngineContext() {
        return engineContext;
    }

    @Override
    public EngineMetadata getEngineMetadata() {
        return engineMetadata;
    }

    @Override
    public synchronized ScrutariSession getScrutariSession() {
        return scrutariSession;
    }

    @Override
    public ScrutariSourceManager getScrutariSourceManager() {
        return scrutariSourceManager;
    }

    @Override
    public FieldVariantManager getFieldVariantManager() {
        return fieldVariantManager;
    }

    @Override
    public AttributeDefManager getAttributeDefManager() {
        return attributeDefManager;
    }

    @Override
    public EngineStorage getEngineStorage() {
        return engineStorage;
    }

    @Override
    public ConfState getConfState() {
        return confState;
    }

    private ScrutariDBName reinitScrutariDB(UpdateLogHandler updateLogHandler, TimeLog timeLog) {
        ScrutariDBName scrutariDbName = ScrutariDBName.newName();
        AliasManager aliasManager = checkBaseURI(updateLogHandler, timeLog);
        engineStorage.cacheObject(scrutariDbName, aliasManager);
        timeLog.log("baseuri_check");
        HistoryDataCoder historyDataCoder = HistoryDataCoderBuilder.build(confDirectory.getInitialCodeMatchArray(aliasManager), engineStorage.getURITree(null, aliasManager));
        DataCoder encapsulatingDataCoder = DataCoderUtils.encapsulate(historyDataCoder);
        timeLog.log("historydatacoder_build");
        AttributeKeyCoder attributeKeyCoder = AttributeDefManagerImpl.toAttributeKeyCoder(confDirectory.getAttributeDefArrayMap());
        timeLog.log("attributekeycoder_build");
        DataCacheWriter dataCacheWriter = engineStorage.getDataCacheWriter(scrutariDbName);
        parseXml(dataCacheWriter, encapsulatingDataCoder, aliasManager, updateLogHandler, timeLog);
        DataCacheReaderFactory dataCacheReaderFactory = dataCacheWriter.endCacheWrite();
        timeLog.log("xml_cache");
        ScrutariDB newScrutariDB = ScrutariDBBuilder.build(scrutariDbName, attributeKeyCoder, dataCacheReaderFactory, engineStorage.getLexieCacheWriter(scrutariDbName), aliasManager, timeLog);
        timeLog.log("scrutaridb_build");
        engineStorage.saveURITree(historyDataCoder.toURITree(), null);
        if (historyDataCoder.hasNewURI()) {
            engineStorage.saveURITree(historyDataCoder.getNewURITree(), scrutariDbName);
        }
        timeLog.log("uritree_saved");
        Set<ScrutariDBName> remainingSet = new HashSet<ScrutariDBName>();
        remainingSet.add(newScrutariDB.getScrutariDBName());
        ScrutariSessionImpl newScrutariSession = ScrutariSessionImpl.newInstance(this, newScrutariDB, aliasManager, confDirectory, timeLog);
        timeLog.log("scrutarisession_build");
        ScrutariSessionImpl oldScrutariSession = scrutariSession;
        if (oldScrutariSession != null) {
            remainingSet.add(oldScrutariSession.getScrutariDB().getScrutariDBName());
        }
        synchronized (this) {
            scrutariSession = newScrutariSession;
        }
        engineStorage.cleanCache(remainingSet);
        return scrutariDbName;
    }

    private AliasManager checkBaseURI(UpdateLogHandler updateLogHandler, TimeLog timeLog) {
        List<ScrutariSource> scrutariSourceList = scrutariSourceManager.getScrutariSourceList();
        AliasManager.Builder aliasManagerBuilder = buildFromSource(scrutariSourceList, updateLogHandler);
        XmlChecker xmlChecker = new XmlChecker();
        BaseURIXmlEngine engine = new BaseURIXmlEngine(updateLogHandler);
        for (ScrutariSource scrutariSource : scrutariSourceList) {
            String scrutariSourceName = scrutariSource.getScrutariSourceDef().getName();
            updateLogHandler.setCurrentScrutariSource(scrutariSourceName);
            timeLog.addSection(scrutariSourceName);
            List<EngineStorage.Data> dataList = engineStorage.getDataList(scrutariSourceName);
            if (dataList.isEmpty()) {
                continue;
            }
            boolean ok = true;
            for (EngineStorage.Data data : dataList) {
                try (InputStream is = data.getInputStream()) {
                    boolean stepDone = xmlChecker.check(is);
                    if (!stepDone) {
                        ok = false;
                    }
                } catch (XmlPullParserException | IOException e) {
                    scrutariSourceManager.setState(scrutariSource, CollectResult.DATA_NOTXML_STATE);
                    updateLogHandler.xmlPullParserException(e);
                    ok = false;
                }
            }
            if (ok) {
                for (EngineStorage.Data data : dataList) {
                    try (InputStream is = data.getInputStream()) {
                        BaseURI baseURI = engine.read(is);
                        if (baseURI != null) {
                            String existingSourceName = aliasManagerBuilder.getSourceName(baseURI);
                            if ((existingSourceName != null) && (!existingSourceName.equals(scrutariSourceName))) {
                                updateLogHandler.addMessage(ScrutariXmlConstants.SEVERE_CATEGORY, LocalisationUtils.toMessage("_ error.existing.baseuri", baseURI));
                            } else {
                                BaseURI currentMain = aliasManagerBuilder.getMainBaseURI(scrutariSourceName);
                                if (currentMain == null) {
                                    aliasManagerBuilder.putMainBaseURI(scrutariSourceName, baseURI);
                                } else {
                                    aliasManagerBuilder.putAliasBaseURI(scrutariSourceName, baseURI);
                                }
                            }
                        }
                    } catch (XmlPullParserException | IOException e) {
                        scrutariSourceManager.setState(scrutariSource, CollectResult.DATA_NOTXML_STATE);
                        updateLogHandler.xmlPullParserException(e);
                    }
                }
            }
        }
        return aliasManagerBuilder.toAliasManager();
    }

    private void parseXml(DataCacheWriter dataCacheWriter, DataCoder dataCoder, AliasManager aliasManager, UpdateLogHandler updateLogHandler, TimeLog timeLog) {
        BaseXmlEngine baseXmlEngine = new BaseXmlEngine();
        List<ScrutariSource> scrutariSourceList = scrutariSourceManager.getScrutariSourceList();
        List<List<EngineStorage.Data>> list = new ArrayList<List<EngineStorage.Data>>();
        for (ScrutariSource scrutariSource : scrutariSourceManager.getScrutariSourceList()) {
            String scrutariSourceName = scrutariSource.getScrutariSourceDef().getName();
            BaseURI baseURI = aliasManager.getMainBaseURI(scrutariSourceName);
            if (baseURI == null) {
                list.add(null);
                continue;
            }
            List<EngineStorage.Data> dataList = engineStorage.getDataList(scrutariSourceName);
            list.add(dataList);
            updateLogHandler.setCurrentScrutariSource(scrutariSourceName);
            timeLog.addSection(scrutariSourceName);
            BaseData.Builder baseDataBuilder = new BaseData.Builder(baseURI);
            int codebase = dataCoder.getCode(baseURI);
            DataValidator dataValidator = confDirectory.getDataValidator();
            if (dataValidator == null) {
                dataValidator = DataValidatorBuilder.NEUTRAL_DATAVALIDATOR;
            }
            Parameters parameters = new Parameters(dataCacheWriter, dataCoder, dataValidator, updateLogHandler, baseURI, codebase);
            boolean first = true;
            for (EngineStorage.Data data : dataList) {
                try (InputStream is = data.getInputStream()) {
                    baseXmlEngine.run(baseDataBuilder, parameters, is, first);
                } catch (XmlPullParserException | IOException e) {
                    scrutariSourceManager.setState(scrutariSource, CollectResult.DATA_NOTXML_STATE);
                    updateLogHandler.xmlPullParserException(e);
                }
                first = false;
            }
            if (engineStorage.hasConfBaseMetadata(scrutariSourceName)) {
                try (InputStream is = engineStorage.getConfBaseMetadataInputstream(scrutariSourceName)) {
                    baseXmlEngine.runMetadataRoot(baseDataBuilder, is, updateLogHandler);
                } catch (XmlPullParserException | IOException e) {
                    updateLogHandler.xmlPullParserException(e);
                }
            }
            BaseData baseData = baseDataBuilder.toBaseData(parameters.getCodeBase(), scrutariSourceName);
            dataCacheWriter.cacheBaseData(baseData);
            timeLog.log("basedata_parsed");
        }
        IndexationXmlEngine indexationXmlEngine = new IndexationXmlEngine(dataCacheWriter, dataCoder, updateLogHandler);
        int dataListSize = list.size();
        for (int i = 0; i < dataListSize; i++) {
            List<EngineStorage.Data> dataList = list.get(i);
            if (dataList == null) {
                continue;
            }
            ScrutariSource scrutariSource = scrutariSourceList.get(i);
            String scrutariSourceName = scrutariSource.getScrutariSourceDef().getName();
            BaseURI baseURI = aliasManager.getMainBaseURI(scrutariSourceName);
            updateLogHandler.setCurrentScrutariSource(scrutariSourceName);
            timeLog.addSection(scrutariSourceName);
            for (EngineStorage.Data data : dataList) {
                try (InputStream is = data.getInputStream()) {
                    indexationXmlEngine.run(is, baseURI);
                } catch (XmlPullParserException | IOException e) {
                    scrutariSourceManager.setState(scrutariSource, CollectResult.DATA_NOTXML_STATE);
                    updateLogHandler.xmlPullParserException(e);
                }
            }
            timeLog.log("indexationdata_parsed");
        }
    }

    private void reloadConfDirectory() {
        try {
            confDirectory.reload();
            checkConfLog(ConfConstants.SOURCES_NAME);
            checkConfLog(ConfConstants.METADATA_NAME);
            checkConfLog(ConfConstants.ATTRIBUTES_NAME);
            checkConfLog(ConfConstants.CATEGORIES_NAME);
            checkConfLog(ConfConstants.URI_CODES_NAME);
            checkConfLog(ConfConstants.OPTIONS_NAME);
            checkConfLog(ConfConstants.VALIDATION_NAME);
            checkConfLog(ConfConstants.FIELDS_NAME);
            confState.setOkState();
            engineMetadata.reload(confDirectory.getEngineMetadata());
            fieldVariantManager.reload(confDirectory.getFieldVariantMap());
            attributeDefManager.reload(confDirectory.getAttributeDefArrayMap());
        } catch (ConfIOException cioe) {
            confState.setGlobalConfigException(cioe);
        }
    }

    private void checkConfLog(String name) {
        ConfFileInfo confFileInfo = confDirectory.getConfFileInfo(name);
        if (!confFileInfo.isHere()) {
            confState.setFileState(name, ConfConstants.FILE_MISSING);
        } else {
            List<CategorizedMessageEntry> messageList = confFileInfo.getMessageList();
            if (!messageList.isEmpty()) {
                int errorType = 0;
                for (CategorizedMessageEntry entry : messageList) {
                    String category = entry.getCategory();
                    if (category.equals(SctLogConstants.MALFORMED_XML_CATEGORY)) {
                        errorType = 2;
                        break;
                    } else if (category.equals(SctLogConstants.SEVERE_CATEGORY)) {
                        errorType = 1;
                    }
                }
                if (errorType == 2) {
                    confState.setFileState(name, ConfConstants.FILE_XML_MALFORMED);
                } else if (errorType == 1) {
                    confState.setFileState(name, ConfConstants.FILE_XML_ERROR);
                } else {
                    confState.setFileState(name, ConfConstants.FILE_XML_WARNING);
                }
            } else {
                confState.setFileState(name, ConfConstants.FILE_OK);
            }
            String logContent = null;
            if (!messageList.isEmpty()) {
                logContent = LogUtils.toXmlString(confFileInfo.getProcessURI(), messageList);
            }
            engineStorage.getLogStorage().log(SctLogConstants.CONF_LOG, name + ".xml", logContent);
        }

    }

    public static AliasManager.Builder buildFromSource(List<ScrutariSource> scrutariSourceList, MessageHandler messageHandler) {
        AliasManager.Builder aliasManagerBuilder = new AliasManager.Builder();
        for (ScrutariSource scrutariSource : scrutariSourceList) {
            ScrutariSourceDef scrutariSourceDef = scrutariSource.getScrutariSourceDef();
            String scrutariSourceName = scrutariSourceDef.getName();
            BaseURI mainBaseURI = scrutariSourceDef.getMainBaseURI();
            if (mainBaseURI != null) {
                if (aliasManagerBuilder.containsBaseURI(mainBaseURI)) {
                    messageHandler.addMessage(ScrutariXmlConstants.SEVERE_CATEGORY, LocalisationUtils.toMessage("_ error.existing.baseuri", mainBaseURI));
                } else {
                    aliasManagerBuilder.putMainBaseURI(scrutariSourceName, mainBaseURI);
                }
            }
            for (BaseURI baseURI : scrutariSourceDef.getAliasList()) {
                if (aliasManagerBuilder.containsBaseURI(baseURI)) {
                    messageHandler.addMessage(ScrutariXmlConstants.SEVERE_CATEGORY, LocalisationUtils.toMessage("_ error.existing.baseuri", mainBaseURI));
                } else {
                    aliasManagerBuilder.putAliasBaseURI(scrutariSourceName, baseURI);
                }
            }
        }
        return aliasManagerBuilder;
    }

}
