/* SctServer - Copyright (c) 2010-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.api.AttributeDefManager;
import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.fieldvariant.FieldVariant;
import fr.exemole.sctserver.json.engineinfo.EngineInfoJson_C;
import fr.exemole.sctserver.json.engineinfo.InfoOptions;
import fr.exemole.sctserver.json.fichedata.FicheDataJson_A;
import fr.exemole.sctserver.json.fichedata.FicheDataJson_B;
import fr.exemole.sctserver.json.fichesearch.CodemotcleArrayJson_A;
import fr.exemole.sctserver.json.fichesearch.GroupTypeJson_A;
import fr.exemole.sctserver.json.fichesearch.GroupTypeJson_B;
import fr.exemole.sctserver.json.fichesearch.InsertOptions;
import fr.exemole.sctserver.json.fichesearch.IntitulesJson_A;
import fr.exemole.sctserver.json.fichesearch.IntitulesJson_B;
import fr.exemole.sctserver.json.fichesearch.ResultGroupJson_A;
import fr.exemole.sctserver.json.fichesearch.ResultGroupJson_B;
import fr.exemole.sctserver.json.fichesearch.SearchMetaJson_A;
import fr.exemole.sctserver.json.fichesearch.SearchMetaJson_B;
import fr.exemole.sctserver.json.motcledata.MotcleDataJson_A;
import fr.exemole.sctserver.json.motcledata.MotcleDataJson_B;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.BaseCheck;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;


/**
 *
 * @author vic
 */
public class FicheSearchResultJsonProducer extends AbstractJsonProducer {

    private final static InfoOptions INFO_OPTIONS;
    private final int version;
    private final ScrutariSession scrutariSession;
    private final AttributeDefManager attributeDefManager;
    private final AttributeKeyCoder attributeKeyCoder;
    private final FicheSearchResult searchResult;
    private final FieldVariant fieldVariant;
    private final InsertOptions insertOptions;
    private final Lang lang;
    private final StartParameters startParameters;

    static {
        INFO_OPTIONS = new InfoOptions();
        INFO_OPTIONS.setAll(true);
        INFO_OPTIONS.setWith(InfoOptions.STATS, false);
    }

    public FicheSearchResultJsonProducer(int version, FicheSearchResult searchResult, ScrutariSession scrutariSession, FieldVariant fieldVariant, InsertOptions insertOptions, StartParameters startParameters, Lang lang) {
        if (lang == null) {
            throw new IllegalArgumentException("lang is null");
        }
        this.version = version;
        this.searchResult = searchResult;
        this.fieldVariant = fieldVariant;
        this.insertOptions = insertOptions;
        this.startParameters = startParameters;
        this.scrutariSession = scrutariSession;
        this.lang = lang;
        this.attributeDefManager = scrutariSession.getEngine().getAttributeDefManager();
        this.attributeKeyCoder = scrutariSession.getScrutariDB().getAttributeKeyCoder();
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariSession.getScrutariDB().openDataAccess();
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        Set<Integer> codecorpusSet = new LinkedHashSet<Integer>();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        jsonWriter.key("ficheSearchResult");
        jsonWriter.object();
        if (insertOptions.isWith(InsertOptions.SEARCHMETA)) {
            if (version >= 3) {
                jsonWriter.key("searchMeta");
                jsonWriter.object();
                SearchMetaJson_B.properties(jsonWriter, dataAccess, searchResult, lang);
                jsonWriter.endObject();
            } else if (version == 2) {
                SearchMetaJson_B.properties(jsonWriter, dataAccess, searchResult, lang);
            } else {
                SearchMetaJson_A.properties(jsonWriter, searchResult, lang);
            }
        }
        if (version >= 1) {
            GroupTypeJson_B.properties(jsonWriter, scrutariSession.getGlobalSearchOptions());
        } else {
            GroupTypeJson_A.properties(jsonWriter, scrutariSession.getGlobalSearchOptions());
        }
        jsonWriter.key("ficheGroupArray");
        jsonWriter.array();
        switch (startParameters.getType()) {
            case StartParameters.NONE_TYPE:
                writeNoneStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.GLOBAL_TYPE:
                writeGlobalStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.IN_ALL_TYPE:
                writeInAllStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.IN_SELECTION_TYPE:
                writeInSelectionStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
        }
        jsonWriter.endArray();
        if (insertOptions.isWith(InsertOptions.MOTCLEARRAY)) {
            if (version >= 3) {
                writeMotcleArrayVersion3(jsonWriter, dataAccess);
            } else {
                writeMotcleArrayVersion1(jsonWriter, dataAccess);
            }
        }
        if (insertOptions.isWith(InsertOptions.ENGINEINFO)) {
            if (version >= 3) {
                EngineInfoJson_C.properties(jsonWriter, dataAccess, scrutariSession, lang, INFO_OPTIONS);
            } else if (version >= 1) {
                IntitulesJson_B.properties(jsonWriter, fieldVariant, dataAccess, lang, codecorpusSet, scrutariSession.getEngine().getAttributeDefManager());
            } else {
                IntitulesJson_A.properties(jsonWriter, fieldVariant, dataAccess, lang, codecorpusSet, scrutariSession.getEngine().getAttributeDefManager());
            }
        }
        jsonWriter.endObject();
        writeWarnings(jsonWriter);
        jsonWriter.endObject();
        dataAccess.close();
    }


    private void writeNoneStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        for (FicheSearchResultGroup ficheSearchResultGroup : searchResult.getFicheSearchResultGroupList()) {
            startGroup(jsonWriter, ficheSearchResultGroup);
            List<FicheSearchResultInfo> infoList = ficheSearchResultGroup.getFicheSearchResultInfoList();
            int ficheByGroupCount = infoList.size();
            for (int j = 0; j < ficheByGroupCount; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = infoList.get(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
            endGroup(jsonWriter);
        }
    }

    private void writeGlobalStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int globalStartIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        boolean limitStop = false;
        int ficheIncrement = 0;
        for (FicheSearchResultGroup ficheSearchResultGroup : searchResult.getFicheSearchResultGroupList()) {
            List<FicheSearchResultInfo> infoList = ficheSearchResultGroup.getFicheSearchResultInfoList();
            int ficheByGroupCount = infoList.size();
            int startIndex = 0;
            if (ficheIncrement == 0) {
                if (ficheByGroupCount < globalStartIndex) {
                    globalStartIndex = globalStartIndex - ficheByGroupCount;
                    continue;
                } else {
                    startIndex = globalStartIndex;
                }
            }
            startGroup(jsonWriter, ficheSearchResultGroup);
            for (int j = startIndex; j < ficheByGroupCount; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = infoList.get(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
                if (withLimit) {
                    ficheIncrement++;
                    if (ficheIncrement == limit) {
                        limitStop = true;
                        break;
                    }
                }
            }
            endGroup(jsonWriter);
            if (limitStop) {
                break;
            }
        }
    }

    private void writeInAllStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int startIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        for (FicheSearchResultGroup ficheSearchResultGroup : searchResult.getFicheSearchResultGroupList()) {
            List<FicheSearchResultInfo> infoList = ficheSearchResultGroup.getFicheSearchResultInfoList();
            int ficheByGroupCount = infoList.size();
            if (ficheByGroupCount < startIndex) {
                continue;
            }
            startGroup(jsonWriter, ficheSearchResultGroup);
            int min = ficheByGroupCount;
            if (withLimit) {
                min = Math.min(ficheByGroupCount, startIndex + limit);
            }
            for (int j = startIndex; j < min; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = infoList.get(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
            endGroup(jsonWriter);
        }
    }

    private void writeInSelectionStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int startIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        for (FicheSearchResultGroup ficheSearchResultGroup : searchResult.getFicheSearchResultGroupList()) {
            if (!startParameters.containsCategory(ficheSearchResultGroup.getCategoryRank())) {
                continue;
            }
            List<FicheSearchResultInfo> infoList = ficheSearchResultGroup.getFicheSearchResultInfoList();
            int ficheByGroupCount = infoList.size();
            if (ficheByGroupCount < startIndex) {
                continue;
            }
            startGroup(jsonWriter, ficheSearchResultGroup);
            int min = ficheByGroupCount;
            if (withLimit) {
                min = Math.min(ficheByGroupCount, startIndex + limit);
            }
            for (int j = startIndex; j < min; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = infoList.get(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
            endGroup(jsonWriter);
        }
    }

    private void startGroup(JSONWriter jsonWriter, FicheSearchResultGroup ficheSearchResultGroup) throws IOException {
        jsonWriter.object();
        if (version >= 1) {
            ResultGroupJson_B.properties(jsonWriter, ficheSearchResultGroup, scrutariSession.getGlobalSearchOptions(), lang);
        } else {
            ResultGroupJson_A.properties(jsonWriter, ficheSearchResultGroup, scrutariSession.getGlobalSearchOptions(), lang);
        }
        jsonWriter.key("ficheArray");
        jsonWriter.array();
    }

    private void endGroup(JSONWriter jsonWriter) throws IOException {
        jsonWriter.endArray();
        jsonWriter.endObject();
    }

    private void writeFicheSearchResultInfo(JSONWriter jsonWriter, FicheSearchResultInfo ficheSearchResultInfo, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int codefiche = ficheSearchResultInfo.getCode();
        FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
        codecorpusSet.add(ficheData.getCodeCorpus());
        BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap, lang);
        jsonWriter.object();
        if (version >= 3) {
            writeFicheVersion3(jsonWriter, ficheData, ficheSearchResultInfo, baseCheck);
        } else {
            writeFicheVersion1(jsonWriter, ficheData, ficheSearchResultInfo, baseCheck);
        }
        jsonWriter.endObject();
    }

    private void writeFicheVersion3(JSONWriter jsonWriter, FicheData ficheData, FicheSearchResultInfo ficheSearchResultInfo, BaseCheck baseCheck) throws IOException {
        FicheDataJson_B.properties(jsonWriter, ficheData, fieldVariant, baseCheck, ficheSearchResultInfo, attributeDefManager, attributeKeyCoder);
        CodemotcleArrayJson_A.properties(jsonWriter, ficheSearchResultInfo);
    }

    private void writeFicheVersion1(JSONWriter jsonWriter, FicheData ficheData, FicheSearchResultInfo ficheSearchResultInfo, BaseCheck baseCheck) throws IOException {
        FicheDataJson_A.properties(jsonWriter, ficheData, fieldVariant, baseCheck, ficheSearchResultInfo, attributeDefManager, attributeKeyCoder);
        CodemotcleArrayJson_A.properties(jsonWriter, ficheSearchResultInfo);
    }

    private void writeMotcleArrayVersion3(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        jsonWriter.key("motcleArray");
        jsonWriter.array();
        for (MotcleSearchResultInfo motcleSearchResultInfo : searchResult.getMotcleSearchResultInfoList()) {
            MotcleData motcleData = dataAccess.getMotcleDataByCode(motcleSearchResultInfo.getCode());
            MotcleDataJson_B.object(jsonWriter, motcleData, fieldVariant, motcleSearchResultInfo, lang);
        }
        jsonWriter.endArray();
    }

    private void writeMotcleArrayVersion1(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        jsonWriter.key("motcleArray");
        jsonWriter.array();
        for (MotcleSearchResultInfo motcleSearchResultInfo : searchResult.getMotcleSearchResultInfoList()) {
            MotcleData motcleData = dataAccess.getMotcleDataByCode(motcleSearchResultInfo.getCode());
            MotcleDataJson_A.object(jsonWriter, motcleData, fieldVariant, motcleSearchResultInfo, lang);
        }
        jsonWriter.endArray();
    }

}
