/* SctServer - Copyright (c) 2015-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.api.AttributeDefManager;
import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.fieldvariant.FieldVariant;
import fr.exemole.sctserver.json.engineinfo.EngineInfoJson_C;
import fr.exemole.sctserver.json.engineinfo.InfoOptions;
import fr.exemole.sctserver.json.fichedata.FicheDataJson_A;
import fr.exemole.sctserver.json.fichedata.FicheDataJson_B;
import fr.exemole.sctserver.json.fichesearch.CodemotcleArrayJson_A;
import fr.exemole.sctserver.json.fichesearch.InsertOptions;
import fr.exemole.sctserver.json.fichesearch.SearchMetaJson_B;
import fr.exemole.sctserver.json.motcledata.MotcleDataJson_A;
import fr.exemole.sctserver.json.motcledata.MotcleDataJson_B;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.tools.util.BaseCheck;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public class GeoJsonProducer extends AbstractJsonProducer {

    private final static InfoOptions INFO_OPTIONS;
    private final int version;
    private final ScrutariSession scrutariSession;
    private final AttributeDefManager attributeDefManager;
    private final AttributeKeyCoder attributeKeyCoder;
    private final FicheSearchResult searchResult;
    private final FieldVariant fieldVariant;
    private final InsertOptions insertOptions;
    private final Lang lang;
    private final StartParameters startParameters;

    static {
        INFO_OPTIONS = new InfoOptions();
        INFO_OPTIONS.setAll(true);
        INFO_OPTIONS.setWith(InfoOptions.STATS, false);
    }

    public GeoJsonProducer(int version, FicheSearchResult searchResult, ScrutariSession scrutariSession, FieldVariant fieldVariant, InsertOptions insertOptions, StartParameters startParameters, Lang lang) {
        this.version = version;
        this.searchResult = searchResult;
        this.fieldVariant = fieldVariant;
        this.insertOptions = insertOptions;
        this.startParameters = startParameters;
        this.scrutariSession = scrutariSession;
        this.lang = lang;
        this.attributeDefManager = scrutariSession.getEngine().getAttributeDefManager();
        this.attributeKeyCoder = scrutariSession.getScrutariDB().getAttributeKeyCoder();
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariSession.getScrutariDB().openDataAccess();
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        Set<Integer> codecorpusSet = new LinkedHashSet<Integer>();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        jsonWriter.key("type");
        jsonWriter.value("FeatureCollection");
        jsonWriter.key("features");
        jsonWriter.array();
        switch (startParameters.getType()) {
            case StartParameters.NONE_TYPE:
                writeNoneStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.GLOBAL_TYPE:
                writeGlobalStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.IN_ALL_TYPE:
                writeInAllStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.IN_SELECTION_TYPE:
                writeInSelectionStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
        }
        jsonWriter.endArray();
        if (version >= 3) {
            if (insertOptions.isWith(InsertOptions.SEARCHMETA)) {
                jsonWriter.key("searchMeta");
                jsonWriter.object();
                SearchMetaJson_B.properties(jsonWriter, dataAccess, searchResult, lang);
                jsonWriter.endObject();
            }
            if (insertOptions.isWith(InsertOptions.MOTCLEARRAY)) {
                writeMotcleArrayVersion3(jsonWriter, dataAccess);
            }
            if (insertOptions.isWith(InsertOptions.ENGINEINFO)) {
                EngineInfoJson_C.properties(jsonWriter, dataAccess, scrutariSession, lang, INFO_OPTIONS);
            }
        } else {
            jsonWriter.key("searchMetadata");
            jsonWriter.object();
            SearchMetaJson_B.properties(jsonWriter, dataAccess, searchResult, lang);
            writeMotcleArrayVersion1(jsonWriter, dataAccess);
            jsonWriter.endObject();
        }
        jsonWriter.endObject();
        dataAccess.close();
    }

    private void writeNoneStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        for (FicheSearchResultGroup ficheSearchResultGroup : searchResult.getFicheSearchResultGroupList()) {
            List<FicheSearchResultInfo> infoList = ficheSearchResultGroup.getFicheSearchResultInfoList();
            int ficheByGroupCount = infoList.size();
            for (int j = 0; j < ficheByGroupCount; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = infoList.get(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
        }
    }

    private void writeGlobalStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int globalStartIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        boolean limitStop = false;
        int ficheIncrement = 0;
        for (FicheSearchResultGroup ficheSearchResultGroup : searchResult.getFicheSearchResultGroupList()) {
            List<FicheSearchResultInfo> infoList = ficheSearchResultGroup.getFicheSearchResultInfoList();
            int ficheByGroupLength = infoList.size();
            int startIndex = 0;
            if (ficheIncrement == 0) {
                if (ficheByGroupLength < globalStartIndex) {
                    globalStartIndex = globalStartIndex - ficheByGroupLength;
                    continue;
                } else {
                    startIndex = globalStartIndex;
                }
            }
            for (int j = startIndex; j < ficheByGroupLength; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = infoList.get(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
                if (withLimit) {
                    ficheIncrement++;
                    if (ficheIncrement == limit) {
                        limitStop = true;
                        break;
                    }
                }
            }
            if (limitStop) {
                break;
            }
        }
    }

    private void writeInAllStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int startIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        for (FicheSearchResultGroup ficheSearchResultGroup : searchResult.getFicheSearchResultGroupList()) {
            List<FicheSearchResultInfo> infoList = ficheSearchResultGroup.getFicheSearchResultInfoList();
            int ficheByGroupLength = infoList.size();
            if (ficheByGroupLength < startIndex) {
                continue;
            }
            int min = ficheByGroupLength;
            if (withLimit) {
                min = Math.min(ficheByGroupLength, startIndex + limit);
            }
            for (int j = startIndex; j < min; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = infoList.get(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
        }
    }

    private void writeInSelectionStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int startIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        for (FicheSearchResultGroup ficheSearchResultGroup : searchResult.getFicheSearchResultGroupList()) {
            if (!startParameters.containsCategory(ficheSearchResultGroup.getCategoryRank())) {
                continue;
            }
            List<FicheSearchResultInfo> infoList = ficheSearchResultGroup.getFicheSearchResultInfoList();
            int ficheByGroupLength = infoList.size();
            if (ficheByGroupLength < startIndex) {
                continue;
            }
            int min = ficheByGroupLength;
            if (withLimit) {
                min = Math.min(ficheByGroupLength, startIndex + limit);
            }
            for (int j = startIndex; j < min; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = infoList.get(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
        }
    }

    private void writeFicheSearchResultInfo(JSONWriter jsonWriter, FicheSearchResultInfo ficheSearchResultInfo, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int codefiche = ficheSearchResultInfo.getCode();
        FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
        if (!ficheData.isWithGeo()) {
            return;
        }
        codecorpusSet.add(ficheData.getCodeCorpus());
        BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap, lang);
        jsonWriter.object();
        jsonWriter.key("type");
        jsonWriter.value("Feature");
        jsonWriter.key("id");
        jsonWriter.value(codefiche);
        jsonWriter.key("geometry");
        jsonWriter.object();
        jsonWriter.key("type");
        jsonWriter.value("Point");
        jsonWriter.key("coordinates");
        jsonWriter.array();
        jsonWriter.value(ficheData.getLongitude().toString());
        jsonWriter.value(ficheData.getLatitude().toString());
        jsonWriter.endArray();
        jsonWriter.endObject();
        jsonWriter.key("properties");
        jsonWriter.object();
        if (version >= 3) {
            writePropertiesVersion3(jsonWriter, ficheData, ficheSearchResultInfo, baseCheck);
        } else {
            writePropertiesVersion1(jsonWriter, ficheData, ficheSearchResultInfo, baseCheck);
        }
        jsonWriter.endObject();
        jsonWriter.endObject();
    }

    private void writePropertiesVersion3(JSONWriter jsonWriter, FicheData ficheData, FicheSearchResultInfo ficheSearchResultInfo, BaseCheck baseCheck) throws IOException {
        FicheDataJson_B.properties(jsonWriter, ficheData, fieldVariant, baseCheck, ficheSearchResultInfo, attributeDefManager, attributeKeyCoder);
        CodemotcleArrayJson_A.properties(jsonWriter, ficheSearchResultInfo);
    }

    private void writePropertiesVersion1(JSONWriter jsonWriter, FicheData ficheData, FicheSearchResultInfo ficheSearchResultInfo, BaseCheck baseCheck) throws IOException {
        FicheDataJson_A.properties(jsonWriter, ficheData, fieldVariant, baseCheck, ficheSearchResultInfo, attributeDefManager, attributeKeyCoder);
        CodemotcleArrayJson_A.properties(jsonWriter, ficheSearchResultInfo);
    }

    private void writeMotcleArrayVersion3(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        jsonWriter.key("motcleArray");
        jsonWriter.array();
        for (MotcleSearchResultInfo motcleSearchResultInfo : searchResult.getMotcleSearchResultInfoList()) {
            MotcleData motcleData = dataAccess.getMotcleDataByCode(motcleSearchResultInfo.getCode());
            MotcleDataJson_B.object(jsonWriter, motcleData, fieldVariant, motcleSearchResultInfo, lang);
        }
        jsonWriter.endArray();
    }

    private void writeMotcleArrayVersion1(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        jsonWriter.key("motcleArray");
        jsonWriter.array();
        for (MotcleSearchResultInfo motcleSearchResultInfo : searchResult.getMotcleSearchResultInfoList()) {
            MotcleData motcleData = dataAccess.getMotcleDataByCode(motcleSearchResultInfo.getCode());
            MotcleDataJson_A.object(jsonWriter, motcleData, fieldVariant, motcleSearchResultInfo, lang);
        }
        jsonWriter.endArray();
    }

}
