/* SctServer - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools;

import java.util.AbstractList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.RandomAccess;
import java.util.Set;
import net.mapeadores.util.attr.AttributeKey;
import net.scrutari.data.DataValidator;


/**
 *
 * @author Vincent Calame
 */
public class DataValidatorBuilder {

    private final static List<Short> EMPTY_LIST = Collections.emptyList();
    private final static List<AttributeKey> EMPTY_ATTRIBUTEKEYLIST = Collections.emptyList();
    public final static DataValidator NEUTRAL_DATAVALIDATOR = new NeutralDataValidator();
    private final Set<AttributeKey> uniqueValueSet = new LinkedHashSet<AttributeKey>();
    private final Set<Short> mandatorySet = new LinkedHashSet<Short>();
    private final Map<Short, StringSizeChecker> checkerMap = new LinkedHashMap<Short, StringSizeChecker>();

    public DataValidatorBuilder() {

    }

    public void setMandatory(short field) {
        mandatorySet.add(field);
    }

    public void setCheck(short field, int min, int max) {
        if (min < 0) {
            min = 0;
        }
        if (max < min) {
            max = min;
        }
        checkerMap.put(field, new StringSizeChecker(min, max));
    }

    public void addUniqueValue(AttributeKey attributeKey) {
        uniqueValueSet.add(attributeKey);
    }

    public DataValidator toDataValidator() {
        List<AttributeKey> uniqueValueList = toAttributeKeyList(uniqueValueSet);
        List<Short> mandatoryList = toShortList(mandatorySet);
        List<Short> checkedList = toShortList(checkerMap.keySet());
        return new InternalDataValidator(new HashSet<Short>(mandatorySet), new HashMap<Short, StringSizeChecker>(checkerMap), uniqueValueList, mandatoryList, checkedList);
    }

    private List<Short> toShortList(Collection<Short> shorts) {
        int size = shorts.size();
        if (size == 0) {
            return EMPTY_LIST;
        }
        return new ShortList(shorts.toArray(new Short[size]));
    }

    private List<AttributeKey> toAttributeKeyList(Collection<AttributeKey> attributes) {
        int size = attributes.size();
        if (size == 0) {
            return EMPTY_ATTRIBUTEKEYLIST;
        }
        return new AttributeKeyList(attributes.toArray(new AttributeKey[size]));
    }


    private static class NeutralDataValidator implements DataValidator {

        private NeutralDataValidator() {

        }

        @Override
        public boolean isMandatory(short field) {
            return false;
        }

        @Override
        public String checkValue(short field, String value) {
            return value;
        }

        @Override
        public List<AttributeKey> getUniqueValueAttributeKeyList() {
            return EMPTY_ATTRIBUTEKEYLIST;
        }

        @Override
        public List<Short> getMandatoryFieldList() {
            return EMPTY_LIST;
        }

        @Override
        public List<Short> getCheckedFieldList() {
            return EMPTY_LIST;
        }

    }


    private static class InternalDataValidator implements DataValidator {

        private final Set<Short> mandatorySet;
        private final Map<Short, StringSizeChecker> checkerMap;
        private final List<AttributeKey> uniqueValueList;
        private final List<Short> mandatoryList;
        private final List<Short> checkedList;

        private InternalDataValidator(Set<Short> mandatorySet, Map<Short, StringSizeChecker> checkerMap, List<AttributeKey> uniqueValueList, List<Short> mandatoryList, List<Short> checkedList) {
            this.mandatorySet = mandatorySet;
            this.checkerMap = checkerMap;
            this.uniqueValueList = uniqueValueList;
            this.mandatoryList = mandatoryList;
            this.checkedList = checkedList;
        }

        @Override
        public boolean isMandatory(short field) {
            return mandatorySet.contains(field);
        }

        @Override
        public String checkValue(short field, String value) {
            if (value == null) {
                return null;
            }
            StringSizeChecker checker = checkerMap.get(field);
            if (checker == null) {
                return value;
            }
            return checker.check(value);
        }

        @Override
        public List<AttributeKey> getUniqueValueAttributeKeyList() {
            return uniqueValueList;
        }

        @Override
        public List<Short> getMandatoryFieldList() {
            return mandatoryList;
        }

        @Override
        public List<Short> getCheckedFieldList() {
            return checkedList;
        }

    }


    private static class StringSizeChecker {

        private final int min;
        private final int max;

        private StringSizeChecker(int min, int max) {
            this.min = min;
            this.max = max;
        }

        private String check(String value) {
            int length = value.length();
            if (length <= max) {
                return value;
            }
            boolean done = false;
            int endIndex = 0;
            for (int i = max; i > min; i--) {
                char carac = value.charAt(i);
                switch (carac) {
                    case '?':
                    case '!':
                    case '.':
                    case '…':
                        endIndex = i;
                        done = true;
                        break;
                }
                if (done) {
                    break;
                }
            }
            if (done) {
                endIndex = testNearEndIndex(endIndex, value);
                return value.substring(0, endIndex + 1);
            } else {
                int stop = checkStop(value);
                return value.substring(0, stop) + '…';
            }
        }

        private int checkStop(String value) {
            for (int i = max; i > min; i--) {
                if (value.charAt(i) == ' ') {
                    return i;
                }
            }
            return min;
        }

        private int testNearEndIndex(int endIndex, String value) {
            int length = value.length();
            int added = 0;
            for (int i = (endIndex + 1); i < length; i++) {
                char carac = value.charAt(i);
                switch (carac) {
                    case '.':
                        added++;
                        if (added > 2) {
                            return endIndex + added;
                        }
                        break;
                    case ')':
                    case ']':
                        added++;
                        return endIndex + added;
                    default:
                        return endIndex + added;
                }
            }
            return endIndex + added;
        }

    }


    private static class ShortList extends AbstractList<Short> implements RandomAccess {

        private final Short[] array;

        private ShortList(Short[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public Short get(int index) {
            return array[index];
        }

    }


    private static class AttributeKeyList extends AbstractList<AttributeKey> implements RandomAccess {

        private final AttributeKey[] array;

        private AttributeKeyList(AttributeKey[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public AttributeKey get(int index) {
            return array[index];
        }

    }

}
