/* SctServer - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.api.fieldvariant.FieldVariant;
import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.fieldvariant.AliasBuilder;
import fr.exemole.sctserver.tools.fieldvariant.FieldVariantBuilder;
import java.util.Map;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class FieldsDOMReader {

    private final Map<String, FieldVariant> map;
    private final MessageHandler messageHandler;

    public FieldsDOMReader(Map<String, FieldVariant> map, MessageHandler messageHandler) {
        this.map = map;
        this.messageHandler = messageHandler;
    }

    public void readFields(Element element, String xpath) {
        xpath = xpath + "/" + element.getTagName();
        DOMUtils.readChildren(element, new RootHandler(xpath));
    }


    private class RootHandler implements ElementHandler {

        private final String rootXpath;

        private RootHandler(String xpath) {
            this.rootXpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            String xpath = rootXpath + "/" + tagName;
            switch (tagName) {
                case "variant":
                    String name = element.getAttribute("name");
                    if (name.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "name");
                    } else if (!StringUtils.isTechnicalName(name, true)) {
                        ErrorUtils.severe(messageHandler, "_ error.wrong.xml.attributevalue", xpath, "name", name);
                    } else if (map.containsKey(name)) {
                        ErrorUtils.severe(messageHandler, "_ error.existing.xml.attributevalue", xpath, "name", name);
                    } else {
                        FieldVariantBuilder fieldVariantBuilder = new FieldVariantBuilder(name);
                        DOMUtils.readChildren(element, new FieldVariantHandler(fieldVariantBuilder, xpath + "[@name='" + name + "']"));
                        map.put(name, fieldVariantBuilder.toFieldVariant());
                    }
                    break;
                default:
                    ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", xpath);
            }
        }

    }


    private class FieldVariantHandler implements ElementHandler {

        private final FieldVariantBuilder fieldVariantBuilder;
        private final String variantXPath;

        private FieldVariantHandler(FieldVariantBuilder fieldVariantBuilder, String variantXPath) {
            this.fieldVariantBuilder = fieldVariantBuilder;
            this.variantXPath = variantXPath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            String xpath = variantXPath + "/" + tagName;
            switch (tagName) {
                case "fiche":
                    String ficheFields = element.getAttribute("fields");
                    if (ficheFields.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "fields");
                    } else {
                        fieldVariantBuilder.parseFicheFields(ficheFields);
                    }
                    break;
                case "motcle":
                    String motcleFields = element.getAttribute("fields");
                    if (motcleFields.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "fields");
                    } else {
                        fieldVariantBuilder.parseMotcleFields(motcleFields);
                    }
                    break;
                case "alias":
                    String aliasName = element.getAttribute("name");
                    String fields = element.getAttribute("fields");
                    boolean ok = true;
                    if (aliasName.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "name");
                        ok = false;
                    }
                    if (fields.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "fields");
                        ok = false;
                    }
                    if (ok) {
                        AliasBuilder aliasBuilder = fieldVariantBuilder.getAliasBuilder(aliasName);
                        aliasBuilder.parseFields(fields);
                        Attr attr = (Attr) element.getAttributes().getNamedItem("separator");
                        if (attr != null) {
                            String separator = attr.getValue();
                            separator = separator.replace("\\n", "\n");
                            aliasBuilder.setSeparator(separator);
                        }
                    }
                    break;
            }
        }

    }

}
