/* SctServer - Copyright (c) 2013-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.xml.feed;

import fr.exemole.sctserver.api.AttributeDefManager;
import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.feed.SctFeedInfo;
import fr.exemole.sctserver.api.feed.SctFeedOptions;
import fr.exemole.sctserver.tools.feed.BaseWrapper;
import fr.exemole.sctserver.tools.feed.CorpusWrapper;
import fr.exemole.sctserver.tools.feed.FicheFeedCodes;
import fr.exemole.sctserver.tools.feed.FicheFeedCodesEngine;
import fr.exemole.sctserver.tools.feed.FicheWrapper;
import fr.exemole.sctserver.tools.feed.ThesaurusWrapper;
import java.io.IOException;
import java.util.List;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.ns.NameSpace;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ScrutariAttributes;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.IndexationCodes;


/**
 *
 * @author Vincent Calame
 */
class FichesEntriesXMLPart extends AbstractEntriesXMLPart {

    private final AttributeDef[] primaryAttributeDefArray;
    private final AttributeDef[] secondaryAttributeDefArray;

    FichesEntriesXMLPart(AppendableXMLWriter xmlWriter, ScrutariSession scrutariSession) {
        super(xmlWriter, scrutariSession);
        AttributeDefManager attributeDefManager = scrutariSession.getEngine().getAttributeDefManager();
        this.primaryAttributeDefArray = toArray(attributeDefManager, AttributeDefManager.PRIMARY_GROUP);
        this.secondaryAttributeDefArray = toArray(attributeDefManager, AttributeDefManager.SECONDARY_GROUP);
    }

    @Override
    public void appendEntries(String feedId, SctFeedInfo sctFeedInfo, DataAccess dataAccess, SctFeedOptions feedOptions) throws IOException {
        Lang lang = feedOptions.getLang();
        int thesaurusCount = dataAccess.getThesaurusCount();
        ThesaurusWrapper[] thesaurusWrapperArray = new ThesaurusWrapper[thesaurusCount];
        for (int i = 0; i < thesaurusCount; i++) {
            thesaurusWrapperArray[i] = ThesaurusWrapper.wrapThesaurus(dataAccess.getThesaurusDataByIndex(i), lang);
        }
        try {
            FicheFeedCodes ficheFeedCodes = FicheFeedCodesEngine.build(scrutariSession, sctFeedInfo, dataAccess, feedOptions);
            if (ficheFeedCodes != null) {
                appendFicheFeedCodes(ficheFeedCodes, dataAccess, thesaurusWrapperArray);
            }
        } catch (ErrorMessageException errorMessageException) {
            CommandMessage errorMessage = errorMessageException.getErrorMessage();
            StringBuilder labelBuf = new StringBuilder();
            Object[] values = errorMessage.getMessageValues();
            int length = values.length;
            for (int i = 0; i < length; i++) {
                if (i > 0) {
                    labelBuf.append(", ");
                }
                labelBuf.append(values[i].toString());
            }
            appendDebugCategory("PARAMETER_ERROR/" + errorMessage.getMessageKey(), labelBuf.toString());
        }
    }

    private AttributeDef[] toArray(AttributeDefManager attributeDefManager, String name) {
        List<AttributeDef> list = attributeDefManager.getAttributeDefList(name);
        return list.toArray(new AttributeDef[list.size()]);
    }

    private void appendDebugCategory(String term, String label) throws IOException {
        startOpenTag("category");
        addAttribute("term", term);
        addAttribute("label", label);
        closeEmptyTag();
    }

    private void appendFicheFeedCodes(FicheFeedCodes ficheFeedCodes, DataAccess dataAccess, ThesaurusWrapper[] thesaurusWrapperArray) throws IOException {
        Lang lang = ficheFeedCodes.getLang();
        int ficheCount = ficheFeedCodes.getFicheCount();
        for (int i = 0; i < ficheCount; i++) {
            FicheWrapper ficheWrapper = ficheFeedCodes.getFicheWrapper(i);
            FicheData ficheData = ficheWrapper.getFicheData();
            CorpusWrapper corpusWrapper = ficheWrapper.getCorpusWrapper();
            BaseWrapper baseWrapper = ficheWrapper.getBaseWrapper();
            openTag("entry");
            String title = ficheData.getTitre();
            if (title == null) {
                title = "?";
            }
            addSimpleElement("title", title);
            addSimpleElement("updated", ficheWrapper.getScrutariDBName().toGmtIsoFormat());
            addSimpleElement("id", ficheData.getFicheURI().toString());
            openTag("author");
            addSimpleElement("name", baseWrapper.getShortLabelString());
            closeTag("author");
            startOpenTag("link");
            addAttribute("href", ficheData.getHref(baseWrapper.getBaseCheck().getCheckedLang()));
            addAttribute("rel", "alternate");
            closeEmptyTag();
            String categoryTerm = corpusWrapper.getCategoryTerm();
            if (categoryTerm != null) {
                startOpenTag("category");
                addAttribute("term", categoryTerm);
                addAttribute("label", corpusWrapper.getCategoryLabel());
                closeEmptyTag();
            }
            startOpenTag("content");
            addAttribute("type", "xhtml");
            endOpenTag();
            startOpenTag("div");
            XMLUtils.appendNameSpaceAttribute(this, NameSpace.XHTML_NAMESPACE);
            endOpenTag();
            appendContent(ficheWrapper, dataAccess, lang, thesaurusWrapperArray);
            closeTag("div");
            closeTag("content");
            closeTag("entry");
        }
    }

    private void appendContent(FicheWrapper ficheWrapper, DataAccess dataAccess, Lang lang, ThesaurusWrapper[] thesaurusWrapperArray) throws IOException {
        FicheData ficheData = ficheWrapper.getFicheData();
        CorpusWrapper corpusWrapper = ficheWrapper.getCorpusWrapper();
        String soustitre = ficheData.getSoustitre();
        if (soustitre != null) {
            openTag("p");
            openTag("em", false);
            addText(soustitre);
            closeTag("em", false);
            closeTag("p", false);
        }
        AttributeHolder attributeHolder = ficheData.getAttributHolder();
        for (AttributeDef attributeDef : primaryAttributeDefArray) {
            Attribute attribute = attributeHolder.getAttributeByKey(attributeDef.getAttributeKey());
            if (attribute != null) {
                appendAttribute(attribute, attributeDef, lang);
            }
        }
        int maxNumber = ficheData.getComplementMaxNumber();
        if (maxNumber > 0) {
            for (int num = 1; num <= maxNumber; num++) {
                String comp = ficheData.getComplementByNumber(num);
                if (comp == null) {
                    continue;
                }
                openTag("p");
                openTag("strong", false);
                addText(corpusWrapper.getComplementLabel(num));
                addText(" : ");
                closeTag("strong", false);
                addText(comp);
                closeTag("p", false);
            }
        }
        for (AttributeDef attributeDef : secondaryAttributeDefArray) {
            Attribute attribute = attributeHolder.getAttributeByKey(attributeDef.getAttributeKey());
            if (attribute != null) {
                appendAttribute(attribute, attributeDef, lang);
            }
        }
        int codefiche = ficheData.getCodeFiche();
        int thesaurusCount = thesaurusWrapperArray.length;
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusWrapper thesaurusWrapper = thesaurusWrapperArray[i];
            IndexationCodes indexationCodes = dataAccess.getIndexationCodes(codefiche, thesaurusWrapper.getCodeThesaurus());
            int indexationCount = indexationCodes.getCodeCount();
            if (indexationCount > 0) {
                openTag("p");
                openTag("strong", false);
                addText(thesaurusWrapper.getLabelString());
                addText(" : ");
                closeTag("strong", false);
                boolean next = false;
                for (int j = 0; j < indexationCount; j++) {
                    int codeMotcle = indexationCodes.getCode(j);
                    MotcleData motcleData = dataAccess.getMotcleDataByCode(codeMotcle);
                    if (motcleData != null) {
                        if (next) {
                            addText(", ");
                        } else {
                            next = true;
                        }
                        Label label = motcleData.getLabelByLang(lang);
                        if (label == null) {
                            if (motcleData.getLabelCount() > 0) {
                                label = motcleData.getLabelByIndex(0);
                            }
                        }
                        if (label != null) {
                            addText(label.getLabelString());
                        }
                    }
                }
                closeTag("p", false);
            }
        }
        openTag("p");
        openTag("strong", false);
        openTag("em", false);
        addText(ficheWrapper.getBaseWrapper().getLongLabelString());
        addText(" – ");
        addText(corpusWrapper.getCorpusLabelString());
        closeTag("em", false);
        closeTag("strong", false);
        closeTag("p", false);
    }

    private void appendAttribute(Attribute attribute, AttributeDef attributeDef, Lang lang) throws IOException {
        Label label = attributeDef.getLabelByLang(lang);
        if (label == null) {
            if (attributeDef.getLabelCount() > 0) {
                label = attributeDef.getLabelByIndex(0);
            }
        }
        String labelString;
        if (label != null) {
            labelString = label.getLabelString();
        } else {
            labelString = attributeDef.getAttributeKey().toString();
        }

        String formatType = ScrutariAttributes.getFormatType(attributeDef);
        if (formatType.equals(ScrutariAttributes.BLOCK_TYPE)) {
            openTag("p");
            openTag("strong", false);
            addText(labelString);
            addText(" : ");
            closeTag("strong", false);
            closeTag("p", false);
            int valueCount = attribute.getValueCount();
            for (int i = 0; i < valueCount; i++) {
                addSimpleElement("p", attribute.getValue(i));
            }
        } else if (formatType.equals(ScrutariAttributes.LIST_TYPE)) {
            openTag("p");
            openTag("strong", false);
            addText(labelString);
            addText(" : ");
            closeTag("strong", false);
            closeTag("p", false);
            openTag("ul");
            int valueCount = attribute.getValueCount();
            for (int i = 0; i < valueCount; i++) {
                addSimpleElement("li", attribute.getValue(i));
            }
            closeTag("ul");
        } else {
            openTag("p");
            openTag("strong", false);
            addText(labelString);
            addText(" : ");
            closeTag("strong", false);
            int valueCount = attribute.getValueCount();
            for (int i = 0; i < valueCount; i++) {
                if (i > 0) {
                    addText(", ");
                }
                addText(attribute.getValue(i));
            }
            closeTag("p", false);
        }
    }

}
