/* SctServer_Directory - Copyright (c) 2014-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.directory.cache;

import fr.exemole.sctserver.api.storage.SctStorageException;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.RandomAccessFile;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.io.DataInputPrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.IndexationData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.caches.CacheIterator;
import net.scrutari.db.api.caches.LexieCacheReaderFactory;
import net.scrutari.lexie.FicheLexification;
import net.scrutari.lexie.LexieOccurrences;
import net.scrutari.lexie.LexieUtils;
import net.scrutari.lexie.MotcleLexification;
import net.scrutari.lexie.ScrutariLexieUnit;


/**
 *
 * @author Vincent Calame
 */
public final class CacheUtils {

    public final static int CACHE_VERSION = 6;
    public final static int BASE_INDEX = 0;
    public final static int CORPUS_INDEX = 1;
    public final static int THESAURUS_INDEX = 2;
    public final static int FICHE_INDEX = 3;
    public final static int MOTCLE_INDEX = 4;
    public final static int INDEXATION_INDEX = 5;
    public final static int LENGTH = 6;

    private CacheUtils() {
    }

    public static CacheIterator<BaseData> newBaseDataIterator(File dataFile) {
        return new BaseDataIterator(dataFile);
    }

    public static CacheIterator<CorpusData> newCorpusDataIterator(File dataFile) {
        return new CorpusDataIterator(dataFile);
    }

    public static CacheIterator<ThesaurusData> newThesaurusDataIterator(File dataFile) {
        return new ThesaurusDataIterator(dataFile);
    }

    public static CacheIterator<FicheData> newFicheDataIterator(File dataFile) {
        return new FicheDataIterator(dataFile);
    }

    public static CacheIterator<MotcleData> newMotcleDataIterator(File dataFile) {
        return new MotcleDataIterator(dataFile);
    }

    public static CacheIterator<IndexationData> newIndexationDataIterator(File dataFile) {
        return new IndexationDataIterator(dataFile);
    }

    public static CacheIterator<LexieCacheReaderFactory.LexieUnitInfo> newLexieUnitInfoIterator(File unitFile) {
        return new LexieUnitInfoIterator(unitFile);
    }

    public static void saveCacheVersion(File scrutaridbCacheDirectory) {
        File versionFile = new File(scrutaridbCacheDirectory.getParent(), "VERSION");
        try (BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(versionFile), "UTF-8"))) {
            bw.write("Version: ");
            bw.write(String.valueOf(CacheUtils.CACHE_VERSION));
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public static int getCacheVersion(File cacheDirectory) {
        File versionFile = new File(cacheDirectory, "VERSION");
        if ((!versionFile.exists()) || (versionFile.isDirectory())) {
            return - 1;
        }
        try (BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(versionFile), "UTF-8"))) {
            int version = -1;
            String ligne;
            while ((ligne = br.readLine()) != null) {
                if (ligne.startsWith("Version:")) {
                    try {
                        version = Integer.parseInt(ligne.substring(8).trim());
                        break;
                    } catch (NumberFormatException nfe) {

                    }
                }
            }
            return version;
        } catch (IOException ioe) {
            return -1;
        }
    }

    public static void checkFichePosition(File cacheFile, Map<Integer, Long> positionMap) throws IOException {
        RandomAccessFile randomAccessFile = new RandomAccessFile(cacheFile, "r");
        DataInputPrimitivesReader primitivesReader = new DataInputPrimitivesReader(randomAccessFile);
        long filelength = randomAccessFile.length();
        long currentPointer = randomAccessFile.getFilePointer();
        while (currentPointer < filelength) {
            FicheData ficheData = FicheData.fromPrimitives(primitivesReader);
            positionMap.put(ficheData.getCodeFiche(), currentPointer);
            currentPointer = randomAccessFile.getFilePointer();
        }
    }

    public static void checkMotclePosition(File cacheFile, Map<Integer, Long> positionMap) throws IOException {
        RandomAccessFile randomAccessFile = new RandomAccessFile(cacheFile, "r");
        DataInputPrimitivesReader primitivesReader = new DataInputPrimitivesReader(randomAccessFile);
        long filelength = randomAccessFile.length();
        long currentPointer = randomAccessFile.getFilePointer();
        while (currentPointer < filelength) {
            MotcleData motcleData = MotcleData.fromPrimitives(primitivesReader);
            positionMap.put(motcleData.getCodeMotcle(), currentPointer);
            currentPointer = randomAccessFile.getFilePointer();
        }
    }

    public static void checkLexificationPosition(File cacheFile, Map<Integer, Long> lexificationPositionMap) throws IOException {
        RandomAccessFile randomAccessFile = new RandomAccessFile(cacheFile, "r");
        DataInputPrimitivesReader primitivesReader = new DataInputPrimitivesReader(randomAccessFile);
        long filelength = randomAccessFile.length();
        long currentPointer = randomAccessFile.getFilePointer();
        while (currentPointer < filelength) {
            int code = primitivesReader.readInt();
            char type = primitivesReader.readChar();
            long position = randomAccessFile.getFilePointer();
            lexificationPositionMap.put(code, position);
            if (type == 'f') {
                FicheLexification.fromPrimitives(primitivesReader);
            } else {
                MotcleLexification.fromPrimitives(primitivesReader);
            }
            currentPointer = randomAccessFile.getFilePointer();
        }
    }

    public static void checkOccurrencePosition(File cacheFile, Map<OccurrencesKey, Long> occurrencesPositionMap) throws IOException {
        RandomAccessFile randomAccessFile = new RandomAccessFile(cacheFile, "r");
        DataInputPrimitivesReader primitivesReader = new DataInputPrimitivesReader(randomAccessFile);
        long filelength = randomAccessFile.length();
        long currentPointer = randomAccessFile.getFilePointer();
        while (currentPointer < filelength) {
            long lexieId = primitivesReader.readLong();
            int subsetCode = primitivesReader.readInt();
            long position = randomAccessFile.getFilePointer();
            occurrencesPositionMap.put(new OccurrencesKey(lexieId, subsetCode), position);
            CacheUtils.lexieOccurrencesListFromPrimitives(primitivesReader);
            currentPointer = randomAccessFile.getFilePointer();
        }
    }

    public static void checkUnitPosition(File cacheFile, Map<Lang, Long> unitPositionMap) throws IOException {
        RandomAccessFile randomAccessFile = new RandomAccessFile(cacheFile, "r");
        DataInputPrimitivesReader primitivesReader = new DataInputPrimitivesReader(randomAccessFile);
        long filelength = randomAccessFile.length();
        long currentPointer = randomAccessFile.getFilePointer();
        while (currentPointer < filelength) {
            Lang lang = Lang.build(primitivesReader.readString());
            long position = randomAccessFile.getFilePointer();
            unitPositionMap.put(lang, position);
            CacheUtils.scrutariLexieUnitListFromPrimitives(primitivesReader);
            currentPointer = randomAccessFile.getFilePointer();
        }
    }

    public static List<ScrutariLexieUnit> scrutariLexieUnitListFromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        int length = primitivesReader.readInt();
        if (length == 0) {
            return LexieUtils.EMPTY_SCRUTARILEXIEUNITLIST;
        } else if (length == 1) {
            return LexieUtils.toScrutariLexieUnitList(ScrutariLexieUnit.fromPrimitives(primitivesReader));
        } else {
            ScrutariLexieUnit[] lexieOccurrencesArray = new ScrutariLexieUnit[length];
            for (int i = 0; i < length; i++) {
                lexieOccurrencesArray[i] = ScrutariLexieUnit.fromPrimitives(primitivesReader);
            }
            return LexieUtils.toScrutariLexieUnitList(lexieOccurrencesArray);
        }
    }

    public static void scrutariLexieUnitListToPrimitives(List<ScrutariLexieUnit> scrutariLexieUnitList, PrimitivesWriter primitivesWriter) throws IOException {
        int length = scrutariLexieUnitList.size();
        primitivesWriter.writeInt(length);
        for (int i = 0; i < length; i++) {
            ScrutariLexieUnit.toPrimitives(scrutariLexieUnitList.get(i), primitivesWriter);
        }
    }

    public static List<LexieOccurrences> lexieOccurrencesListFromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        int length = primitivesReader.readInt();
        if (length == 0) {
            return LexieUtils.EMPTY_LEXIEOCCURRENCESLIST;
        } else if (length == 1) {
            return LexieUtils.wrap(LexieOccurrences.fromPrimitives(primitivesReader));
        } else {
            LexieOccurrences[] lexieOccurrencesArray = new LexieOccurrences[length];
            for (int i = 0; i < length; i++) {
                lexieOccurrencesArray[i] = LexieOccurrences.fromPrimitives(primitivesReader);
            }
            return LexieUtils.wrap(lexieOccurrencesArray);
        }
    }

    public static void lexieOccurencesListToPrimitives(List<LexieOccurrences> lexieOccurencesList, PrimitivesWriter primitivesWriter) throws IOException {
        int length = lexieOccurencesList.size();
        primitivesWriter.writeInt(length);
        for (int i = 0; i < length; i++) {
            LexieOccurrences.toPrimitives(lexieOccurencesList.get(i), primitivesWriter);
        }
    }


    private static class BaseDataIterator implements CacheIterator<BaseData> {

        private final RandomAccessFile dataRandomAccessFile;
        private final DataInputPrimitivesReader dataPrimitivesReader;

        private BaseDataIterator(File dataFile) {
            try {
                this.dataRandomAccessFile = new RandomAccessFile(dataFile, "r");
                this.dataPrimitivesReader = new DataInputPrimitivesReader(dataRandomAccessFile);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public boolean hasNext() {
            try {
                return (dataRandomAccessFile.getFilePointer() < dataRandomAccessFile.length());
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public BaseData next() {
            if (!hasNext()) {
                throw new NoSuchElementException();
            }
            try {
                return BaseData.fromPrimitives(dataPrimitivesReader);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public void close() {
            try {
                dataRandomAccessFile.close();
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

    }


    private static class CorpusDataIterator implements CacheIterator<CorpusData> {

        private final RandomAccessFile dataRandomAccessFile;
        private final DataInputPrimitivesReader dataPrimitivesReader;

        private CorpusDataIterator(File dataFile) {
            try {
                this.dataRandomAccessFile = new RandomAccessFile(dataFile, "r");
                this.dataPrimitivesReader = new DataInputPrimitivesReader(dataRandomAccessFile);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public boolean hasNext() {
            try {
                return (dataRandomAccessFile.getFilePointer() < dataRandomAccessFile.length());
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public CorpusData next() {
            if (!hasNext()) {
                throw new NoSuchElementException();
            }
            try {
                return CorpusData.fromPrimitives(dataPrimitivesReader);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public void close() {
            try {
                dataRandomAccessFile.close();
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

    }


    private static class ThesaurusDataIterator implements CacheIterator<ThesaurusData> {

        private final RandomAccessFile dataRandomAccessFile;
        private final DataInputPrimitivesReader dataPrimitivesReader;

        private ThesaurusDataIterator(File dataFile) {
            try {
                this.dataRandomAccessFile = new RandomAccessFile(dataFile, "r");
                this.dataPrimitivesReader = new DataInputPrimitivesReader(dataRandomAccessFile);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public boolean hasNext() {
            try {
                return (dataRandomAccessFile.getFilePointer() < dataRandomAccessFile.length());
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public ThesaurusData next() {
            if (!hasNext()) {
                throw new NoSuchElementException();
            }
            try {
                return ThesaurusData.fromPrimitives(dataPrimitivesReader);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public void close() {
            try {
                dataRandomAccessFile.close();
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

    }


    private static class FicheDataIterator implements CacheIterator<FicheData> {

        private final RandomAccessFile dataRandomAccessFile;
        private final DataInputPrimitivesReader dataPrimitivesReader;

        private FicheDataIterator(File dataFile) {
            try {
                this.dataRandomAccessFile = new RandomAccessFile(dataFile, "r");
                this.dataPrimitivesReader = new DataInputPrimitivesReader(dataRandomAccessFile);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public boolean hasNext() {
            try {
                return (dataRandomAccessFile.getFilePointer() < dataRandomAccessFile.length());
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public FicheData next() {
            if (!hasNext()) {
                throw new NoSuchElementException();
            }
            try {
                return FicheData.fromPrimitives(dataPrimitivesReader);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public void close() {
            try {
                dataRandomAccessFile.close();
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

    }


    private static class MotcleDataIterator implements CacheIterator<MotcleData> {

        private final RandomAccessFile dataRandomAccessFile;
        private final DataInputPrimitivesReader dataPrimitivesReader;

        private MotcleDataIterator(File dataFile) {
            try {
                this.dataRandomAccessFile = new RandomAccessFile(dataFile, "r");
                this.dataPrimitivesReader = new DataInputPrimitivesReader(dataRandomAccessFile);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public boolean hasNext() {
            try {
                return (dataRandomAccessFile.getFilePointer() < dataRandomAccessFile.length());
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public MotcleData next() {
            if (!hasNext()) {
                throw new NoSuchElementException();
            }
            try {
                return MotcleData.fromPrimitives(dataPrimitivesReader);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public void close() {
            try {
                dataRandomAccessFile.close();
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

    }


    private static class IndexationDataIterator implements CacheIterator<IndexationData> {

        private final RandomAccessFile dataRandomAccessFile;
        private final DataInputPrimitivesReader dataPrimitivesReader;

        private IndexationDataIterator(File dataFile) {
            try {
                this.dataRandomAccessFile = new RandomAccessFile(dataFile, "r");
                this.dataPrimitivesReader = new DataInputPrimitivesReader(dataRandomAccessFile);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public boolean hasNext() {
            try {
                return (dataRandomAccessFile.getFilePointer() < dataRandomAccessFile.length());
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public IndexationData next() {
            if (!hasNext()) {
                throw new NoSuchElementException();
            }
            try {
                return IndexationData.fromPrimitives(dataPrimitivesReader);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public void close() {
            try {
                dataRandomAccessFile.close();
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

    }


    private static class LexieUnitInfoIterator implements CacheIterator<LexieCacheReaderFactory.LexieUnitInfo> {

        private final RandomAccessFile dataRandomAccessFile;
        private final DataInputPrimitivesReader dataPrimitivesReader;

        private LexieUnitInfoIterator(File unitFile) {
            try {
                this.dataRandomAccessFile = new RandomAccessFile(unitFile, "r");
                this.dataPrimitivesReader = new DataInputPrimitivesReader(dataRandomAccessFile);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public boolean hasNext() {
            try {
                return (dataRandomAccessFile.getFilePointer() < dataRandomAccessFile.length());
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public LexieCacheReaderFactory.LexieUnitInfo next() {
            if (!hasNext()) {
                throw new NoSuchElementException();
            }
            try {
                Lang lang = Lang.build(dataPrimitivesReader.readString());
                List<ScrutariLexieUnit> scrutariLexieUnitList = scrutariLexieUnitListFromPrimitives(dataPrimitivesReader);
                return new InternalLexieUnitInfo(lang, scrutariLexieUnitList);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public void close() {
            try {
                dataRandomAccessFile.close();
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

    }


    private static class InternalLexieUnitInfo implements LexieCacheReaderFactory.LexieUnitInfo {

        private final Lang lang;
        private final List<ScrutariLexieUnit> scrutariLexieUnitList;

        private InternalLexieUnitInfo(Lang lang, List<ScrutariLexieUnit> scrutariLexieUnitList) {
            this.lang = lang;
            this.scrutariLexieUnitList = scrutariLexieUnitList;
        }

        @Override
        public Lang getLang() {
            return lang;
        }

        @Override
        public List<ScrutariLexieUnit> getScrutariLexieUnitList() {
            return scrutariLexieUnitList;
        }

    }

}
