/* UtilLib - Copyright (c) 2011-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import java.text.Collator;
import java.text.RuleBasedCollator;
import java.util.List;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.collation.CollationUnit;


/**
 *
 * @author Vincent Calame
 */
public class TextTestEngine {

    private final RuleBasedCollator collator;
    private final short mode;
    private final Tester tester;
    private final Lang lang;
    private CollatedStringTestUnit[] excludingTestUnitArray;
    private CollatedStringTestUnit[] includingTestUnitArray;
    private int allExcludedMask = 0;
    private int allIncludedMask = 0;

    private TextTestEngine(Condition condition, Lang lang) {
        this.lang = lang;
        collator = (RuleBasedCollator) Collator.getInstance(lang.toLocale());
        collator.setStrength(Collator.PRIMARY);
        List<TextTest> excludingList = condition.getExcludingTextTestList();
        int excludingCount = excludingList.size();
        if (excludingCount > 0) {
            excludingTestUnitArray = new CollatedStringTestUnit[excludingCount];
            for (int i = 0; i < excludingCount; i++) {
                excludingTestUnitArray[i] = CollatedStringTestUnit.newInstance(excludingList.get(i), collator);
            }
            allExcludedMask = getMask(excludingCount);
        }
        List<TextTest> includingList = condition.getIncludingTextTestList();
        int includingCount = includingList.size();
        if (includingCount > 0) {
            includingTestUnitArray = new CollatedStringTestUnit[includingCount];
            for (int i = 0; i < includingCount; i++) {
                includingTestUnitArray[i] = CollatedStringTestUnit.newInstance(includingList.get(i), collator);
            }
            allIncludedMask = getMask(includingCount);
        }
        mode = condition.getMode();
        if (mode == ConditionsConstants.MODE_AND) {
            tester = new AndTester();
        } else {
            tester = new OrTester();
        }
    }

    public Lang getLang() {
        return lang;
    }

    public void start() {
        tester.clear();
    }

    public boolean canStop() {
        return tester.canStop();
    }

    public boolean addString(String s) {
        String collatedString = CollationUnit.collate(s, collator);
        tester.addString(collatedString);
        if (tester.canStop()) {
            return true;
        } else {
            return false;
        }
    }

    public boolean getResult() {
        return tester.getResult();
    }

    public boolean isSelected(String s) {
        String collatedString;
        if ((s == null) || (s.length() == 0)) {
            collatedString = "";
        } else {
            collatedString = CollationUnit.collate(s, collator);
        }
        if (mode == ConditionsConstants.MODE_AND) {
            return isAndAccepted(collatedString);
        } else {
            return isOrAccepted(collatedString);
        }
    }

    private boolean isAndAccepted(String collatedString) {
        if (includingTestUnitArray != null) {
            for (CollatedStringTestUnit testUnit : includingTestUnitArray) {
                if (!testUnit.accept(collatedString)) {
                    return false;
                }
            }
        }
        if (excludingTestUnitArray != null) {
            for (CollatedStringTestUnit testUnit : excludingTestUnitArray) {
                if (!testUnit.accept(collatedString)) {
                    return false;
                }
            }
        }
        return true;
    }

    private boolean isOrAccepted(String collatedString) {
        if (includingTestUnitArray != null) {
            for (CollatedStringTestUnit testUnit : includingTestUnitArray) {
                if (testUnit.accept(collatedString)) {
                    return true;
                }
            }
        }
        if (excludingTestUnitArray != null) {
            for (CollatedStringTestUnit testUnit : excludingTestUnitArray) {
                if (testUnit.accept(collatedString)) {
                    return true;
                }
            }
        }
        return false;
    }

    public static TextTestEngine newInstance(Condition condition, Lang lang) {
        if (condition == null) {
            throw new IllegalArgumentException("condition is null");
        }
        if (lang == null) {
            throw new IllegalArgumentException("lang is null");
        }
        return new TextTestEngine(condition, lang);
    }

    private static int getMask(int count) {
        return ((int) Math.pow(2, count) - 1);
    }

    private static int getPow(int i) {
        return ((int) Math.pow(2, i));
    }


    private abstract class Tester {

        protected boolean doExclude;
        protected boolean doInclude;

        Tester() {
            init();
        }

        void init() {
            doExclude = (excludingTestUnitArray != null);
            doInclude = (includingTestUnitArray != null);
        }

        abstract void addString(String collatedString);

        abstract boolean canStop();

        abstract boolean getResult();

        abstract void clear();

    }


    private class OrTester extends Tester {

        private boolean success = false;
        private int failedExcludeMask;

        OrTester() {
        }

        @Override
        public void addString(String collatedString) {
            if (success) {
                return;
            }
            if (doInclude) {
                int length = includingTestUnitArray.length;
                for (int i = 0; i < length; i++) {
                    if (includingTestUnitArray[i].accept(collatedString)) {
                        success = true;
                        return;
                    }
                }
            }
            if (doExclude) {
                int length = excludingTestUnitArray.length;
                for (int i = 0; i < length; i++) {
                    int pow = getPow(i);
                    if ((pow & failedExcludeMask) == 0) {
                        if (!excludingTestUnitArray[i].accept(collatedString)) {
                            failedExcludeMask = failedExcludeMask | pow;
                            if (failedExcludeMask == allExcludedMask) {
                                doExclude = false;
                            }
                        }
                    }
                }
            }
        }

        @Override
        boolean canStop() {
            return (success == true);
        }

        @Override
        boolean getResult() {
            if (success) {
                return true;
            }
            if (allExcludedMask == 0) {
                return false;
            }
            return (failedExcludeMask != allExcludedMask);
        }

        @Override
        void clear() {
            init();
            success = false;
            failedExcludeMask = 0;
        }

    }


    private class AndTester extends Tester {

        private boolean failure;
        private int successIncludeMask;

        @Override
        void addString(String collatedString) {
            if (failure) {
                return;
            }
            if (doInclude) {
                int length = includingTestUnitArray.length;
                for (int i = 0; i < length; i++) {
                    int pow = getPow(i);
                    if ((pow & successIncludeMask) == 0) {
                        if (includingTestUnitArray[i].accept(collatedString)) {
                            successIncludeMask = successIncludeMask | pow;
                            if (successIncludeMask == allIncludedMask) {
                                doInclude = false;
                            }
                        }
                    }
                }
            }
            if (doExclude) {
                int length = excludingTestUnitArray.length;
                for (int i = 0; i < length; i++) {
                    if (!excludingTestUnitArray[i].accept(collatedString)) {
                        failure = true;
                        return;
                    }
                }
            }
        }

        @Override
        boolean canStop() {
            return (failure == true);
        }

        @Override
        boolean getResult() {
            if (failure) {
                return false;
            }
            if (allIncludedMask == 0) {
                return true;
            }
            return (successIncludeMask == allIncludedMask);
        }

        @Override
        void clear() {
            init();
            failure = false;
            successIncludeMask = 0;
        }

    }

}
